@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkCanvPoly.c,v 1.2 1998/09/14 18:23:05 stanton Exp $
+ * RCS: @(#) $Id: tkCanvPoly.c,v 1.5 2000/02/01 11:41:09 hobbs Exp $
  */
 
 #include <stdio.h>
@@ -34,9 +34,6 @@ typedef struct PolygonItem  {
 				 * X-coords are even-valued indices, y-coords
 				 * are corresponding odd-valued indices. */
     int joinStyle;		/* Join style for outline */
-    Tk_Tile fillTile;		/* Tile for filling rectangle/oval. */
-    Tk_Tile activeFillTile;	/* Tile for filling rectangle/oval if state is active. */
-    Tk_Tile disabledFillTile;	/* Tile for filling rectangle/oval if state is disabled. */
     Tk_TSOffset tsoffset;
     XColor *fillColor;		/* Foreground color for polygon. */
     XColor *activeFillColor;	/* Foreground color for polygon if state is active. */
@@ -61,29 +58,25 @@ static Tk_CustomOption smoothOption = {
     TkSmoothPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption stateOption = {
-    (Tk_OptionParseProc *) Tk_StateParseProc,
-    Tk_StatePrintProc, (ClientData) 2
+    (Tk_OptionParseProc *) TkStateParseProc,
+    TkStatePrintProc, (ClientData) 2
 };
 static Tk_CustomOption tagsOption = {
     (Tk_OptionParseProc *) Tk_CanvasTagsParseProc,
     Tk_CanvasTagsPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption dashOption = {
-    (Tk_OptionParseProc *) Tk_CanvasDashParseProc,
-    Tk_CanvasDashPrintProc, (ClientData) NULL
-};
-static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc, (ClientData) NULL
+    (Tk_OptionParseProc *) TkCanvasDashParseProc,
+    TkCanvasDashPrintProc, (ClientData) NULL
 };
 static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
+    (Tk_OptionParseProc *) TkOffsetParseProc,
+    TkOffsetPrintProc,
     (ClientData) (TK_OFFSET_RELATIVE|TK_OFFSET_INDEX)
 };
 static Tk_CustomOption pixelOption = {
-    (Tk_OptionParseProc *) Tk_PixelParseProc,
-    Tk_PixelPrintProc, (ClientData) NULL
+    (Tk_OptionParseProc *) TkPixelParseProc,
+    TkPixelPrintProc, (ClientData) NULL
 };
 
 static Tk_ConfigSpec configSpecs[] = {
@@ -99,15 +92,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-activeoutlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.activeStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-activeoutlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.activeTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_BITMAP, "-activestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, activeFillStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-activetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, activeFillTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-activewidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(PolygonItem, outline.activeWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -129,15 +116,9 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-disabledoutlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.disabledStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledoutlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.disabledTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_BITMAP, "-disabledstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, disabledFillStipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledtile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, disabledFillTile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-disabledwidth", (char *) NULL, (char *) NULL,
 	"0.0", Tk_Offset(PolygonItem, outline.disabledWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -157,9 +138,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_BITMAP, "-outlinestipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(PolygonItem, outline.stipple),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-outlinetile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, outline.tile),
-	TK_CONFIG_NULL_OK, &tileOption},
     {TK_CONFIG_CUSTOM, "-smooth", (char *) NULL, (char *) NULL,
 	"0", Tk_Offset(PolygonItem, smooth),
 	TK_CONFIG_DONT_SET_DEFAULT, &smoothOption},
@@ -172,11 +150,6 @@ static Tk_ConfigSpec configSpecs[] = {
 	(char *) NULL, Tk_Offset(PolygonItem, fillStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-tags", (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
-    {TK_CONFIG_CUSTOM, "-tile", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(PolygonItem, fillTile),
-	TK_CONFIG_NULL_OK, &tileOption},
-    {TK_CONFIG_STRING, "-updatecommand", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(Tk_Item, updateCmd), TK_CONFIG_NULL_OK},
     {TK_CONFIG_CUSTOM, "-width", (char *) NULL, (char *) NULL,
 	"1.0", Tk_Offset(PolygonItem, outline.width),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -192,10 +165,10 @@ static void		ComputePolygonBbox _ANSI_ARGS_((Tk_Canvas canvas,
 			    PolygonItem *polyPtr));
 static int		ConfigurePolygon _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
-			    char **argv, int flags));
+			    Tcl_Obj *CONST argv[], int flags));
 static int		CreatePolygon _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, struct Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int argc, Tcl_Obj *CONST argv[]));
 static void		DeletePolygon _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr,  Display *display));
 static void		DisplayPolygon _ANSI_ARGS_((Tk_Canvas canvas,
@@ -206,7 +179,7 @@ static int		GetPolygonIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *obj, int *indexPtr));
 static int		PolygonCoords _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr,
-			    int argc, char **argv));
+			    int argc, Tcl_Obj *CONST argv[]));
 static void		PolygonDeleteCoords _ANSI_ARGS_((Tk_Canvas canvas,
 			    Tk_Item *itemPtr, int first, int last));
 static void		PolygonInsert _ANSI_ARGS_((Tk_Canvas canvas,
@@ -237,7 +210,7 @@ Tk_ItemType tkPolygonType = {
     PolygonCoords,			/* coordProc */
     DeletePolygon,			/* deleteProc */
     DisplayPolygon,			/* displayProc */
-    TK_ITEM_VISITOR_SUPPORT|TK_CONFIG_OBJS,/* flags */
+    TK_CONFIG_OBJS,			/* flags */
     PolygonToPoint,			/* pointProc */
     PolygonToArea,			/* areaProc */
     PolygonToPostscript,		/* postscriptProc */
@@ -249,10 +222,6 @@ Tk_ItemType tkPolygonType = {
     (Tk_ItemInsertProc *) PolygonInsert,/* insertProc */
     PolygonDeleteCoords,		/* dTextProc */
     (Tk_ItemType *) NULL,		/* nextPtr */
-    (Tk_ItemBboxProc *) ComputePolygonBbox,/* bboxProc */
-    Tk_Offset(Tk_VisitorType, visitPolygon), /* acceptProc */
-    (Tk_ItemGetCoordProc *) NULL,	/* getCoordProc */
-    (Tk_ItemSetCoordProc *) NULL	/* setCoordProc */
 };
 
 /*
@@ -274,7 +243,7 @@ Tk_ItemType tkPolygonType = {
  * Results:
  *	A standard Tcl return value.  If an error occurred in
  *	creating the item, then an error message is left in
- *	interp->result;  in this case itemPtr is
+ *	the interp's result;  in this case itemPtr is
  *	left uninitialized, so it can be safely freed by the
  *	caller.
  *
@@ -291,7 +260,7 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
     Tk_Item *itemPtr;			/* Record to hold new item;  header
 					 * has been initialized by caller. */
     int argc;				/* Number of arguments in argv. */
-    char **argv;			/* Arguments describing polygon. */
+    Tcl_Obj *CONST argv[];		/* Arguments describing polygon. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
     int i;
@@ -306,9 +275,6 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
     polyPtr->pointsAllocated = 0;
     polyPtr->coordPtr = NULL;
     polyPtr->joinStyle = JoinRound;
-    polyPtr->fillTile = NULL;
-    polyPtr->activeFillTile = NULL;
-    polyPtr->disabledFillTile = NULL;
     polyPtr->tsoffset.flags = 0;
     polyPtr->tsoffset.xoffset = 0;
     polyPtr->tsoffset.yoffset = 0;
@@ -360,7 +326,7 @@ CreatePolygon(interp, canvas, itemPtr, argc, argv)
  *	on what it does.
  *
  * Results:
- *	Returns TCL_OK or TCL_ERROR, and sets interp->result.
+ *	Returns TCL_OK or TCL_ERROR, and sets the interp's result.
  *
  * Side effects:
  *	The coordinates for the given item may be changed.
@@ -376,7 +342,7 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 					 * read or modified. */
     int argc;				/* Number of coordinates supplied in
 					 * argv. */
-    char **argv;			/* Array of coordinates: x1, y1,
+    Tcl_Obj *CONST argv[];		/* Array of coordinates: x1, y1,
 					 * x2, y2, ... */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
@@ -396,7 +362,7 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 	return TCL_OK;
     }
     if (argc == 1) {
-	if (Tcl_ListObjGetElements(interp, (Tcl_Obj *) argv[0], &argc,
+	if (Tcl_ListObjGetElements(interp, argv[0], &argc,
 		(Tcl_Obj ***) &argv) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -423,7 +389,7 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
 	    polyPtr->pointsAllocated = numPoints+1;
 	}
 	for (i = argc-1; i >= 0; i--) {
-	    if (Tk_CanvasGetCoordFromObj(interp, canvas, (Tcl_Obj *) argv[i],
+	    if (Tk_CanvasGetCoordFromObj(interp, canvas, argv[i],
 		    &polyPtr->coordPtr[i]) != TCL_OK) {
 		return TCL_ERROR;
 	    }
@@ -457,7 +423,7 @@ PolygonCoords(interp, canvas, itemPtr, argc, argv)
  *
  * Results:
  *	A standard Tcl result code.  If an error occurs, then
- *	an error message is left in interp->result.
+ *	an error message is left in the interp's result.
  *
  * Side effects:
  *	Configuration information, such as colors and stipple
@@ -472,7 +438,7 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     Tk_Canvas canvas;		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr;		/* Polygon item to reconfigure. */
     int argc;			/* Number of elements in argv.  */
-    char **argv;		/* Arguments describing things to configure. */
+    Tcl_Obj *CONST argv[];	/* Arguments describing things to configure. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
@@ -480,14 +446,12 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     GC newGC;
     unsigned long mask;
     Tk_Window tkwin;
-    Tk_Tile tile;
     XColor *color;
     Pixmap stipple;
     Tk_State state;
-    Pixmap pixmap;
 
     tkwin = Tk_CanvasTkwin(canvas);
-    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, argv,
+    if (Tk_ConfigureWidget(interp, tkwin, configSpecs, argc, (char **) argv,
 	    (char *) polyPtr, flags|TK_CONFIG_OBJS) != TCL_OK) {
 	return TCL_ERROR;
     }
@@ -500,11 +464,9 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     state = itemPtr->state;
 
     if (polyPtr->outline.activeWidth > polyPtr->outline.width ||
-	    polyPtr->outline.activeDash.number > 0 ||
-	    polyPtr->outline.activeTile != NULL ||
+	    polyPtr->outline.activeDash.number != 0 ||
 	    polyPtr->outline.activeColor != NULL ||
 	    polyPtr->outline.activeStipple != None ||
-	    polyPtr->activeFillTile != NULL ||
 	    polyPtr->activeFillColor != NULL ||
 	    polyPtr->activeFillStipple != None) {
 	itemPtr->redraw_flags |= TK_ITEM_STATE_DEPENDANT;
@@ -534,13 +496,9 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
     }
     polyPtr->outline.gc = newGC;
 
-    tile = polyPtr->fillTile;
     color = polyPtr->fillColor;
     stipple = polyPtr->fillStipple;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (polyPtr->activeFillTile!=NULL) {
-	    tile = polyPtr->activeFillTile;
-	}
 	if (polyPtr->activeFillColor!=NULL) {
 	    color = polyPtr->activeFillColor;
 	}
@@ -548,9 +506,6 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
 	    stipple = polyPtr->activeFillStipple;
 	}
     } else if (state==TK_STATE_DISABLED) {
-	if (polyPtr->disabledFillTile!=NULL) {
-	    tile = polyPtr->disabledFillTile;
-	}
 	if (polyPtr->disabledFillColor!=NULL) {
 	    color = polyPtr->disabledFillColor;
 	}
@@ -559,16 +514,7 @@ ConfigurePolygon(interp, canvas, itemPtr, argc, argv, flags)
 	}
     }
 
-    Tk_SetTileCanvasItem(polyPtr->fillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(polyPtr->activeFillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(polyPtr->disabledFillTile, canvas, (Tk_Item *) NULL);
-    Tk_SetTileCanvasItem(tile, canvas, itemPtr);
-
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	newGC = Tk_GetGC(tkwin, GCTile|GCFillStyle, &gcValues);
-    } else if (color == NULL) {
+    if (color == NULL) {
 	newGC = None;
     } else {
 	gcValues.foreground = color->pixel;
@@ -629,15 +575,6 @@ DeletePolygon(canvas, itemPtr, display)
     if (polyPtr->coordPtr != NULL) {
 	ckfree((char *) polyPtr->coordPtr);
     }
-    if (polyPtr->fillTile != NULL) {
-	Tk_FreeTile(polyPtr->fillTile);
-    }
-    if (polyPtr->activeFillTile != NULL) {
-	Tk_FreeTile(polyPtr->activeFillTile);
-    }
-    if (polyPtr->disabledFillTile != NULL) {
-	Tk_FreeTile(polyPtr->disabledFillTile);
-    }
     if (polyPtr->fillColor != NULL) {
 	Tk_FreeColor(polyPtr->fillColor);
     }
@@ -946,7 +883,6 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
     Tk_State state = itemPtr->state;
-    Tk_Tile tile = polyPtr->fillTile;
     Pixmap stipple = polyPtr->fillStipple;
     double linewidth = polyPtr->outline.width;
 
@@ -966,9 +902,6 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	if (polyPtr->activeFillStipple != None) {
 	    stipple = polyPtr->activeFillStipple;
 	}
-	if (polyPtr->activeFillTile != NULL) {
-	    tile = polyPtr->activeFillTile;
-	}
     } else if (state==TK_STATE_DISABLED) {
 	if (polyPtr->outline.disabledWidth>0.0) {
 	    linewidth = polyPtr->outline.disabledWidth;
@@ -976,9 +909,6 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	if (polyPtr->disabledFillStipple != None) {
 	    stipple = polyPtr->disabledFillStipple;
 	}
-	if (polyPtr->disabledFillTile != NULL) {
-	    tile = polyPtr->disabledFillTile;
-	}
     }
     /*
      * If we're stippling then modify the stipple offset in the GC.  Be
@@ -986,16 +916,12 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
      * read-only.
      */
 
-    if ((tile != NULL) || (stipple != None)) {
+    if (stipple != None) {
 	Tk_TSOffset *tsoffset = &polyPtr->tsoffset;
 	int w=0; int h=0;
 	int flags = tsoffset->flags;
 	if (!(flags & TK_OFFSET_INDEX) && (flags & (TK_OFFSET_CENTER|TK_OFFSET_MIDDLE))) {
-	    if (tile != NULL) {
-		Tk_SizeOfTile(tile, &w, &h);
-	    } else {
-		Tk_SizeOfBitmap(display, stipple, &w, &h);
-	    }
+	    Tk_SizeOfBitmap(display, stipple, &w, &h);
 	    if (flags & TK_OFFSET_CENTER) {
 		w /= 2;
 	    } else {
@@ -1015,7 +941,7 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
     }
     Tk_ChangeOutlineGC(canvas, itemPtr, &(polyPtr->outline));
 
-    if (polyPtr->numPoints < 3) {
+    if(polyPtr->numPoints < 3) {
 	short x,y;
 	int intLineWidth = (int) (linewidth + 0.5);
 	if (intLineWidth < 1) {
@@ -1023,8 +949,10 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	}
 	Tk_CanvasDrawableCoords(canvas, polyPtr->coordPtr[0],
 		    polyPtr->coordPtr[1], &x,&y);
-	XFillArc(display, drawable, polyPtr->outline.gc, x - intLineWidth/2,
-		    y - intLineWidth/2, intLineWidth+1, intLineWidth+1, 0, 64*360);
+	XFillArc(display, drawable, polyPtr->outline.gc,
+		x - intLineWidth/2, y - intLineWidth/2,
+		(unsigned int)intLineWidth+1, (unsigned int)intLineWidth+1,
+		0, 64*360);
     } else if (!polyPtr->smooth || polyPtr->numPoints < 4) {
 	TkFillPolygon(canvas, polyPtr->coordPtr, polyPtr->numPoints,
 		    display, drawable, polyPtr->fillGC, polyPtr->outline.gc);
@@ -1063,7 +991,7 @@ DisplayPolygon(canvas, itemPtr, display, drawable, x, y, width, height)
 	}
     }
     Tk_ResetOutlineGC(canvas, itemPtr, &(polyPtr->outline));
-    if ((tile != NULL) && (stipple != None)) {
+    if (stipple != None) {
 	XSetTSOrigin(display, polyPtr->fillGC, 0, 0);
     }
 }
@@ -1302,7 +1230,7 @@ PolygonToPoint(canvas, itemPtr, pointPtr)
     int changedMiterToBevel;	/* Non-zero means that a mitered corner
 				 * had to be treated as beveled after all
 				 * because the angle was < 11 degrees. */
-    int width;
+    double width;
     Tk_State state = itemPtr->state;
 
     bestDist = 1.0e36;
@@ -1310,14 +1238,14 @@ PolygonToPoint(canvas, itemPtr, pointPtr)
     if(state == TK_STATE_NULL) {
 	state = ((TkCanvas *)canvas)->canvas_state;
     }
-    width = (int) polyPtr->outline.width;
+    width = polyPtr->outline.width;
     if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
 	if (polyPtr->outline.activeWidth>width) {
-	    width = (int) polyPtr->outline.activeWidth;
+	    width = polyPtr->outline.activeWidth;
 	}
     } else if (state==TK_STATE_DISABLED) {
 	if (polyPtr->outline.disabledWidth>0.0) {
-	    width = (int) polyPtr->outline.disabledWidth;
+	    width = polyPtr->outline.disabledWidth;
 	}
     }
     radius = width/2.0;
@@ -1734,8 +1662,8 @@ GetPolygonIndex(interp, canvas, itemPtr, obj, indexPtr)
     int *indexPtr;		/* Where to store converted index. */
 {
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
-    int length;
-    char *string = Tcl_GetStringFromObj(obj, &length);
+    size_t length;
+    char *string = Tcl_GetStringFromObj(obj, (int *) &length);
 
     if (string[0] == 'e') {
 	if (strncmp(string, "end", length) == 0) {
@@ -1847,7 +1775,7 @@ TranslatePolygon(canvas, itemPtr, deltaX, deltaY)
  * Results:
  *	The return value is a standard Tcl result.  If an error
  *	occurs in generating Postscript then an error message is
- *	left in interp->result, replacing whatever used
+ *	left in the interp's result, replacing whatever used
  *	to be there.  If no error occurs, then Postscript for the
  *	item is appended to the result.
  *
@@ -1868,8 +1796,8 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 					 * collect font information;  0 means
 					 * final Postscript is being created. */
 {
-    char string[100], *style;
     PolygonItem *polyPtr = (PolygonItem *) itemPtr;
+    char *style;
     XColor *color;
     XColor *fillColor;
     Pixmap stipple;
@@ -1922,8 +1850,8 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 	    fillStipple = polyPtr->disabledFillStipple;
 	}
     }
-
     if (polyPtr->numPoints==2) {
+	char string[128];
 	sprintf(string, "%.15g %.15g translate %.15g %.15g",
 		polyPtr->coordPtr[0], Tk_CanvasPsY(canvas, polyPtr->coordPtr[1]),
 		width/2.0, width/2.0);
@@ -1943,6 +1871,7 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
 	}
 	return TCL_OK;
     }
+
     /*
      * Fill the area of the polygon.
      */
@@ -1977,6 +1906,7 @@ PolygonToPostscript(interp, canvas, itemPtr, prepass)
      */
 
     if (color != NULL) {
+
 	if (!polyPtr->smooth || !polyPtr->smooth->postscriptProc) {
 	    Tk_CanvasPsPath(interp, canvas, polyPtr->coordPtr,
 		polyPtr->numPoints);
