@@ -21,11 +21,6 @@
 #include "tkInt.h"
 #include <signal.h>
 
-#ifndef USE_TCL_STUBS
-extern TclStubs tclStubs;
-#define tclStubsPtr (&tclStubs)
-#endif
-
 /*
  * Make sure that Tcl_MainLoop always references the stub pointer, not
  * the real function.
@@ -35,6 +30,11 @@ extern TclStubs tclStubs;
 #undef Tcl_SetMainLoop
 #define Tcl_SetMainLoop ((void (*) _ANSI_ARGS_((VOID *))) ((tclStubsPtr)->tcl_SetMainLoop))
 
+#ifndef USE_TCL_STUBS
+extern TclStubs tclStubs;
+#define tclStubsPtr (&tclStubs)
+#endif
+
 /*
  * There's a potential problem if a handler is deleted while it's
  * current (i.e. its procedure is executing), since Tk_HandleEvent
@@ -975,6 +975,7 @@ Tk_QueueWindowEvent(eventPtr, position)
          * if state of the event is zero.
          */
         eventPtr->xkey.state = 0;
+	dispPtr->isComposed = 1;
     }
 #endif /* TK_USE_INPUT_METHODS && I18N_IMPROVE */
 
@@ -1164,11 +1165,6 @@ DelayedMotionProc(clientData)
  *--------------------------------------------------------------
  */
 
-#ifndef USE_TCL_STUBS
-extern TclStubs tclStubs;
-#   define Tcl_MainLoop tclStubs.tcl_MainLoop
-#endif
-
 void
 Tk_MainLoop()
 {
