@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkFrame.c,v 1.4 1999/08/10 05:06:01 jingham Exp $
+ * RCS: @(#) $Id: tkFrame.c,v 1.6 2000/02/10 08:52:32 hobbs Exp $
  */
 
 #include "default.h"
@@ -62,9 +62,6 @@ typedef struct {
     int highlightWidth;		/* Width in pixels of highlight to draw
 				 * around widget when it has the focus.
 				 * 0 means don't draw a highlight. */
-    Tk_Tile tile;		/* tiling */
-    Tk_TSOffset tsoffset;	/* offset for tiling */
-    GC tileGC;			/* GC for tiling */
     XColor *highlightBgColorPtr;
 				/* Color for drawing traversal highlight
 				 * area when highlight is off. */
@@ -100,22 +97,6 @@ typedef struct {
 #define REDRAW_PENDING		1
 #define GOT_FOCUS		4
 
-/*
- * Custom option for handling "-tile"
- */
-
-static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) TkTileParseProc,
-    TkTilePrintProc,
-    (ClientData) NULL
-};
-
-static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) TkOffsetParseProc,
-    TkOffsetPrintProc,
-    (ClientData) NULL
-};
-
 /*
  * The following flag bits are used so that there can be separate
  * defaults for some configuration options for frames and toplevels.
@@ -162,9 +143,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-menu", "menu", "Menu",
 	DEF_TOPLEVEL_MENU, Tk_Offset(Frame, menuName),
 	TOPLEVEL|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
-	Tk_Offset(Frame, tsoffset),BOTH|TK_CONFIG_DONT_SET_DEFAULT,
-	&offsetOption},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
 	DEF_FRAME_RELIEF, Tk_Offset(Frame, relief), BOTH},
     {TK_CONFIG_STRING, "-screen", "screen", "Screen",
@@ -173,8 +151,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_FRAME_TAKE_FOCUS, Tk_Offset(Frame, takeFocus),
 	BOTH|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(Frame, tile),BOTH|TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_STRING, "-use", "use", "Use",
 	DEF_FRAME_USE, Tk_Offset(Frame, useThis), TOPLEVEL|TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-visual", "visual", "Visual",
@@ -193,6 +169,9 @@ static Tk_ConfigSpec configSpecs[] = {
 static int		ConfigureFrame _ANSI_ARGS_((Tcl_Interp *interp,
 			    Frame *framePtr, int objc, Tcl_Obj *CONST objv[],
 			    int flags));
+static int		CreateFrame _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, int objc, Tcl_Obj *CONST argv[],
+			    int toplevel, char *appName));
 static void		DestroyFrame _ANSI_ARGS_((char *memPtr));
 static void		DisplayFrame _ANSI_ARGS_((ClientData clientData));
 static void		FrameCmdDeletedProc _ANSI_ARGS_((
@@ -230,7 +209,7 @@ Tk_FrameObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    return TkCreateFrame(clientData, interp, objc, objv, 0, (char *) NULL);
+    return CreateFrame(clientData, interp, objc, objv, 0, (char *) NULL);
 }
 
 int
@@ -241,13 +220,13 @@ Tk_ToplevelObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    return TkCreateFrame(clientData, interp, objc, objv, 1, (char *) NULL);
+    return CreateFrame(clientData, interp, objc, objv, 1, (char *) NULL);
 }
 
 /*
  *--------------------------------------------------------------
  *
- * TkFrameCreate --
+ * TkCreateFrame --
  *
  *	This procedure is invoked to process the "frame" and "toplevel"
  *	Tcl commands;  it is also invoked directly by Tk_Init to create
@@ -262,9 +241,39 @@ Tk_ToplevelObjCmd(clientData, interp, objc, objv)
  *
  *--------------------------------------------------------------
  */
+extern TkStubs tkStubs;
 
 int
-TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
+TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
+    ClientData clientData;	/* Main window associated with interpreter.
+				 * If we're called by Tk_Init to create a
+				 * new application, then this is NULL. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    char **argv;		/* Argument strings. */
+    int toplevel;		/* Non-zero means create a toplevel window,
+				 * zero means create a frame. */
+    char *appName;		/* Should only be non-NULL if clientData is
+				 * NULL:  gives the base name to use for the
+				 * new application. */
+{
+    int result, i;
+    Tcl_Obj **objv = (Tcl_Obj **) ckalloc((argc+1) * sizeof(Tcl_Obj **));
+    for (i=0; i<argc; i++) {
+	objv[i] = Tcl_NewStringObj(argv[i], -1);
+	Tcl_IncrRefCount(objv[i]);
+    }
+    objv[argc] = NULL;
+    result = CreateFrame(clientData, interp, argc, objv, toplevel, appName);
+    for (i=0; i<argc; i++) {
+	Tcl_DecrRefCount(objv[i]);
+    }
+    ckfree((char *) objv);
+    return result;
+}
+
+static int
+CreateFrame(clientData, interp, objc, objv, toplevel, appName)
     ClientData clientData;	/* Main window associated with interpreter.
 				 * If we're called by Tk_Init to create a
 				 * new application, then this is NULL. */
@@ -281,10 +290,15 @@ TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
     Frame *framePtr;
     Tk_Window new;
     char *className, *screenName, *visualName, *colormapName, *arg, *useOption;
-    int i, c, length, depth;
+    int i, c, depth;
+    size_t length;
     unsigned int mask;
     Colormap colormap;
     Visual *visual;
+    if ((Tcl_InitStubs(interp, "8.1", 0) == NULL) ||
+	    (Tcl_PkgProvideEx(interp, "Tk", TK_VERSION, (ClientData) &tkStubs) != TCL_OK)) {
+	return TCL_ERROR;
+    }
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?options?");
@@ -301,7 +315,7 @@ TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
     className = colormapName = screenName = visualName = useOption = NULL;
     colormap = None;
     for (i = 2; i < objc; i += 2) {
-	arg = Tcl_GetStringFromObj(objv[i], &length);
+	arg = Tcl_GetStringFromObj(objv[i], (int *) &length);
 	if (length < 2) {
 	    continue;
 	}
@@ -343,7 +357,8 @@ TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
 	screenName = (toplevel) ? "" : NULL;
     }
     if (tkwin != NULL) {
-	new = Tk_CreateWindowFromPath(interp, tkwin, Tcl_GetString(objv[1]), screenName);
+	new = Tk_CreateWindowFromPath(interp, tkwin, Tcl_GetString(objv[1]),
+		screenName);
     } else {
 	/*
 	 * We were called from Tk_Init;  create a new application.
@@ -427,11 +442,6 @@ TkCreateFrame(clientData, interp, objc, objv, toplevel, appName)
     framePtr->borderWidth = 0;
     framePtr->relief = TK_RELIEF_FLAT;
     framePtr->highlightWidth = 0;
-    framePtr->tile = NULL;
-    framePtr->tsoffset.flags = 0;
-    framePtr->tsoffset.xoffset = 0;
-    framePtr->tsoffset.yoffset = 0;
-    framePtr->tileGC = NULL;
     framePtr->highlightBgColorPtr = NULL;
     framePtr->highlightColorPtr = NULL;
     framePtr->width = 0;
@@ -548,7 +558,7 @@ FrameWidgetObjCmd(clientData, interp, objc, objv)
 	     */
 
 	    for (i = 2; i < objc; i++) {
-		char *arg = Tcl_GetStringFromObj(objv[i], &length);
+		char *arg = Tcl_GetStringFromObj(objv[i], (int *) &length);
 		if (length < 2) {
 		    continue;
 		}
@@ -611,40 +621,12 @@ DestroyFrame(memPtr)
 
     Tk_FreeOptions(configSpecs, (char *) framePtr, framePtr->display,
 	    framePtr->mask);
-    if (framePtr->tile != NULL) {
-	Tk_FreeTile(framePtr->tile);
-    }
-    if (framePtr->tileGC != None) {
-	Tk_FreeGC(framePtr->display, framePtr->tileGC);
-    }
     if (framePtr->colormap != None) {
 	Tk_FreeColormap(framePtr->display, framePtr->colormap);
     }
     ckfree((char *) framePtr);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;			/* Not used */
-{
-    register Frame *framePtr = (Frame *) clientData;
-
-    ConfigureFrame(framePtr->interp, framePtr, 0, NULL, 0);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -676,9 +658,6 @@ ConfigureFrame(interp, framePtr, objc, objv, flags)
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     char *oldMenuName;
-    GC new = None;
-    XGCValues gcValues;
-    Pixmap pixmap;
     
     /*
      * Need the old menubar name for the menu code to delete it.
@@ -705,23 +684,12 @@ ConfigureFrame(interp, framePtr, objc, objv, flags)
 		framePtr->menuName);
     }
     
-    Tk_SetTileChangedProc(framePtr->tile, TileChangedProc,
-	(ClientData)framePtr, (Tk_Item *) NULL);
-    if ((pixmap = Tk_PixmapOfTile(framePtr->tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	new = Tk_GetGC(framePtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else if (framePtr->border != NULL) {
+    if (framePtr->border != NULL) {
 	Tk_SetBackgroundFromBorder(framePtr->tkwin, framePtr->border);
     } else {
 	Tk_SetWindowBackgroundPixmap(framePtr->tkwin, None);
     }
 
-    if (framePtr->tileGC != None) {
-	Tk_FreeGC(framePtr->display, framePtr->tileGC);
-    }
-    framePtr->tileGC = new;
-
     if (framePtr->highlightWidth < 0) {
 	framePtr->highlightWidth = 0;
     }
@@ -777,49 +745,6 @@ DisplayFrame(clientData)
 	return;
     }
 
-    if (framePtr->tileGC != NULL) {
-	int borderWidth = framePtr->highlightWidth;
-	if (framePtr->border != NULL) {
-	    borderWidth += framePtr->borderWidth;
-	}
-	if ((Tk_Width(tkwin) > (2 * borderWidth)) &&
-		(Tk_Height(tkwin) > (2 * borderWidth))) {
-	    Tk_SetTileOrigin(tkwin, framePtr->tileGC, 0, 0);
-	    if (framePtr->tsoffset.flags) {
-		int w=0; int h=0;
-		if (framePtr->tsoffset.flags &
-			(TK_OFFSET_CENTER|TK_OFFSET_MIDDLE)) {
-		    Tk_SizeOfTile(framePtr->tile, &w, &h);
-		}
-		if (framePtr->tsoffset.flags & TK_OFFSET_LEFT) {
-		    w = 0;
-		} else if (framePtr->tsoffset.flags & TK_OFFSET_RIGHT) {
-		    w = Tk_Width(tkwin);
-		} else {
-		    w = (Tk_Width(tkwin) - w) / 2;
-		}
-		if (framePtr->tsoffset.flags & TK_OFFSET_TOP) {
-		    h = 0;
-		} else if (framePtr->tsoffset.flags & TK_OFFSET_BOTTOM) {
-		    h = Tk_Height(tkwin);
-		} else {
-		    h = (Tk_Height(tkwin) - h) / 2;
-		}
-		XSetTSOrigin(framePtr->display, framePtr->tileGC, w , h);
-	    } else {
-	        Tk_SetTileOrigin(tkwin, framePtr->tileGC,
-			framePtr->tsoffset.xoffset,
-			framePtr->tsoffset.yoffset);
-	    }
-	    XFillRectangle(framePtr->display, Tk_WindowId(tkwin),
-		    framePtr->tileGC, borderWidth, borderWidth,
-		    Tk_Width(tkwin) - 2 * borderWidth,
-		    Tk_Height(tkwin) - 2 * borderWidth);
-	    XSetTSOrigin(framePtr->display, framePtr->tileGC, 0, 0);
-	    drawFunction = Tk_Draw3DRectangle;
-	}
-    }
-
     if (framePtr->border != NULL) {
 	drawFunction(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
@@ -1055,6 +980,9 @@ TkInstallFrameMenu(tkwin)
     if (winPtr->mainPtr != NULL) {
 	Frame *framePtr;
 	framePtr = (Frame*) winPtr->instanceData;
+	if (framePtr == NULL) {
+	    panic("TkInstallFrameMenu couldn't get frame pointer");
+	}
 	TkpMenuNotifyToplevelCreate(winPtr->mainPtr->interp, 
 		framePtr->menuName);
     }
