@@ -13,11 +13,9 @@
  *	   Department of Computer Science,
  *	   Australian National University.
  *
- * RCS: @(#) $Id: tkImgPPM.c,v 1.5 1999/07/01 00:39:44 redman Exp $
+ * RCS: @(#) $Id: tkImgPPM.c,v 1.7 1999/12/21 23:55:11 hobbs Exp $
  */
 
-#define USE_OLD_IMAGE
-
 #include "tkInt.h"
 #include "tkPort.h"
 
@@ -40,15 +38,16 @@
  */
 
 static int		FileMatchPPM _ANSI_ARGS_((Tcl_Channel chan,
-			    char *fileName, char *formatString,
-			    int *widthPtr, int *heightPtr));
+			    CONST char *fileName, Tcl_Obj *format,
+			    int *widthPtr, int *heightPtr,
+			    Tcl_Interp *interp));
 static int		FileReadPPM  _ANSI_ARGS_((Tcl_Interp *interp,
-			    Tcl_Channel chan, char *fileName,
-			    char *formatString, Tk_PhotoHandle imageHandle,
+			    Tcl_Channel chan, CONST char *fileName,
+			    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 			    int destX, int destY, int width, int height,
 			    int srcX, int srcY));
 static int		FileWritePPM _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *fileName, char *formatString,
+			    CONST char *fileName, Tcl_Obj *format,
 			    Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat tkImgFmtPPM = {
@@ -88,13 +87,14 @@ static int		ReadPPMFileHeader _ANSI_ARGS_((Tcl_Channel chan,
  */
 
 static int
-FileMatchPPM(chan, fileName, formatString, widthPtr, heightPtr)
+FileMatchPPM(chan, fileName, format, widthPtr, heightPtr, interp)
     Tcl_Channel chan;		/* The image file, open for reading. */
-    char *fileName;		/* The name of the image file. */
-    char *formatString;		/* User-specified format string, or NULL. */
+    CONST char *fileName;	/* The name of the image file. */
+    Tcl_Obj *format;		/* User-specified format, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw PPM file. */
+    Tcl_Interp *interp;		/* not used */
 {
     int dummy;
 
@@ -122,12 +122,12 @@ FileMatchPPM(chan, fileName, formatString, widthPtr, heightPtr)
  */
 
 static int
-FileReadPPM(interp, chan, fileName, formatString, imageHandle, destX, destY,
+FileReadPPM(interp, chan, fileName, format, imageHandle, destX, destY,
 	width, height, srcX, srcY)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     Tcl_Channel chan;		/* The image file, open for reading. */
-    char *fileName;		/* The name of the image file. */
-    char *formatString;		/* User-specified format string, or NULL. */
+    CONST char *fileName;	/* The name of the image file. */
+    Tcl_Obj *format;		/* User-specified format, or NULL. */
     Tk_PhotoHandle imageHandle;	/* The photo image to write into. */
     int destX, destY;		/* Coordinates of top-left pixel in
 				 * photo image to be written to. */
@@ -255,10 +255,10 @@ FileReadPPM(interp, chan, fileName, formatString, imageHandle, destX, destY,
  */
 
 static int
-FileWritePPM(interp, fileName, formatString, blockPtr)
+FileWritePPM(interp, fileName, format, blockPtr)
     Tcl_Interp *interp;
-    char *fileName;
-    char *formatString;
+    CONST char *fileName;
+    Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     Tcl_Channel chan;
@@ -267,7 +267,7 @@ FileWritePPM(interp, fileName, formatString, blockPtr)
     unsigned char *pixelPtr, *pixLinePtr;
     char header[16 + TCL_INTEGER_SPACE * 2];
 
-    chan = Tcl_OpenFileChannel(interp, fileName, "w", 0666);
+    chan = Tcl_OpenFileChannel(interp, (char *) fileName, "w", 0666);
     if (chan == NULL) {
 	return TCL_ERROR;
     }
@@ -276,10 +276,6 @@ FileWritePPM(interp, fileName, formatString, blockPtr)
 	    != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (Tcl_SetChannelOption(interp, chan, "-encoding", "binary")
-	    != TCL_OK) {
-	return TCL_ERROR;
-    }
     
     sprintf(header, "P6\n%d %d\n255\n", blockPtr->width, blockPtr->height);
     Tcl_Write(chan, header, -1);
@@ -355,7 +351,7 @@ ReadPPMFileHeader(chan, widthPtr, heightPtr, maxIntensityPtr)
 {
 #define BUFFER_SIZE 1000
     char buffer[BUFFER_SIZE];
-    int i, numFields, firstInLine;
+    int i, numFields;
     int type = 0;
     char c;
 
@@ -367,7 +363,6 @@ ReadPPMFileHeader(chan, widthPtr, heightPtr, maxIntensityPtr)
     if (Tcl_Read(chan, &c, 1) != 1) {
 	return 0;
     }
-    firstInLine = 1;
     i = 0;
     for (numFields = 0; numFields < 4; numFields++) {
 	/*
@@ -376,7 +371,6 @@ ReadPPMFileHeader(chan, widthPtr, heightPtr, maxIntensityPtr)
 
 	while (1) {
 	    while (isspace(UCHAR(c))) {
-		firstInLine = (c == '\n');
 		if (Tcl_Read(chan, &c, 1) != 1) {
 		    return 0;
 		}
@@ -389,7 +383,6 @@ ReadPPMFileHeader(chan, widthPtr, heightPtr, maxIntensityPtr)
 		    return 0;
 		}
 	    } while (c != '\n');
-	    firstInLine = 1;
 	}
 
 	/*
@@ -409,7 +402,6 @@ ReadPPMFileHeader(chan, widthPtr, heightPtr, maxIntensityPtr)
 	    buffer[i] = ' ';
 	    i++;
 	}
-	firstInLine = 0;
     }
     done:
     buffer[i] = 0;
