@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: $Id: tkInt.h,v 1.21 2000/01/21 03:54:41 hobbs Exp $ 
+ * RCS: $Id: tkInt.h,v 1.23 2000/04/19 01:06:51 ericm Exp $ 
  */
 
 #ifndef _TKINT
@@ -481,19 +481,6 @@ typedef struct TkDisplay {
 
 #ifdef TK_USE_INPUT_METHODS
     XIM inputMethod;		/* Input method for this display */
-#ifdef I18N_IMPROVE
-    Tcl_Encoding imEncoding;	/* Tcl encoding when the first Tcl
-				 * interp was created.
-				 * For encoding conversion from
-				 * XmbLookupString() to UTF.
-				 */
-    XIC lastFocusedIC;		/* The last focused input context on
-				 * the display.
-				 */
-#ifdef XNDestroyCallback
-    XIMCallback destroyCallback;
-#endif /* XNDestroyCallback */
-#endif /* I18N_IMPROVE */
 #endif /* TK_USE_INPUT_METHODS */
     Tcl_HashTable winTable;	/* Maps from X window ids to TkWindow ptrs. */
 
@@ -512,6 +499,24 @@ typedef struct TkDisplay {
     int warpX;
     int warpY;
     int useInputMethods;	/* Whether to use input methods */
+#ifdef TK_USE_INPUT_METHODS
+#ifdef I18N_IMPROVE
+    Tcl_Encoding imEncoding;	/* Tcl encoding when the first Tcl
+				 * interp was created.
+				 * For encoding conversion from
+				 * XmbLookupString() to UTF.
+				 */
+    XIC lastFocusedIC;		/* The last focused input context on
+				 * the display.
+				 */
+    int isComposed;             /* 1 if the latest KeyPress event is a
+                                 * "Compose message" from IM server,
+                                 * otherwise 0. */
+#ifdef XNDestroyCallback
+    XIMCallback destroyCallback;
+#endif /* XNDestroyCallback */
+#endif /* I18N_IMPROVE */
+#endif /* TK_USE_INPUT_METHODS */
 } TkDisplay;
 
 /*
@@ -803,6 +808,13 @@ typedef struct TkWindow {
      */
 
     struct TkWindowPrivate *privatePtr;
+#ifdef TK_USE_INPUT_METHODS
+#ifdef I18N_IMPROVE
+    Tcl_DString composedDStr;	/* To prepare successive "%A" in bind
+				 * script, save a string from IM
+				 * server. */
+#endif /* I18N_IMPROVE */
+#endif /* TK_USE_INPUT_METHODS */
 } TkWindow;
 
 /*
@@ -863,6 +875,15 @@ extern TkDisplay *tkDisplayList;
 
 #define EMBEDDED_APP_WANTS_FOCUS (NotifyNormal + 20)
 
+/*
+ * The following special modifier mask bits are defined, to indicate
+ * logical modifiers such as Meta and Alt that may float among the
+ * actual modifier bits.
+ */
+
+#define META_MASK	(AnyModifier<<1)
+#define ALT_MASK	(AnyModifier<<2)
+
 /*
  * Miscellaneous variables shared among Tk modules but not exported
  * to the outside world:
@@ -886,6 +907,8 @@ extern int			tkSendSerial;
 # define TCL_STORAGE_CLASS DLLEXPORT
 #endif
 
+extern TCL_STORAGE_CLASS TkStubs tkStubs;
+
 /*
  * Internal procedures shared among Tk modules but not exported
  * to the outside world:
@@ -969,8 +992,9 @@ EXTERN int              Tk_MessageBoxObjCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Obj *CONST objv[]));
 EXTERN int		Tk_MessageCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
-EXTERN int		Tk_OptionCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
+EXTERN int		Tk_OptionObjCmd _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, int objc,
+	                    Tcl_Obj *CONST objv[]));
 EXTERN int		Tk_PackCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 EXTERN int		Tk_PlaceCmd _ANSI_ARGS_((ClientData clientData,
@@ -1018,8 +1042,6 @@ EXTERN int		TkCreateMenuCmd _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN int		TkDeadAppCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 
-EXTERN int		TkpTestembedCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, char **argv));
 EXTERN int		TkCanvasGetCoordObj _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tcl_Obj *obj,
 			    double *doublePtr));
@@ -1097,14 +1119,12 @@ EXTERN void		TkpDestroyIC _ANSI_ARGS_((TkWindow *winPtr, int needDestroy));
 EXTERN int		Tk_ImconfigureObjCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
-#endif
+#endif /* TK_USE_INPUT_METHODS && I18N_IMPROVE */
 
 # undef TCL_STORAGE_CLASS
 # define TCL_STORAGE_CLASS
 
 EXTERN int	TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN void	TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
-			    int devId, char *buffer, long size));
 
 # undef TCL_STORAGE_CLASS
 # define TCL_STORAGE_CLASS DLLIMPORT
