@@ -13,12 +13,12 @@
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
- * Copyright (c) 1999 by Scriptics Corporation.
+ * Copyright (c) 1998-2000 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkScale.c,v 1.8 1999/04/21 21:53:27 rjohnson Exp $
+ * RCS: @(#) $Id: tkScale.c,v 1.12 2000/04/14 08:33:16 hobbs Exp $
  */
 
 #include "tkPort.h"
@@ -53,9 +53,6 @@ static Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
 	DEF_SCALE_ACTIVE_BG_COLOR, -1, Tk_Offset(TkScale, activeBorder),
 	0, (ClientData) DEF_SCALE_ACTIVE_BG_MONO, 0},
-/*    {TK_CONFIG_CUSTOM, "-activetile", "activeTile", "Tile", (char *) NULL,
-	Tk_Offset(TkScale, activeTile), TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},*/
     {TK_OPTION_BORDER, "-background", "background", "Background",
 	DEF_SCALE_BG_COLOR, -1, Tk_Offset(TkScale, bgBorder),
 	0, (ClientData) DEF_SCALE_BG_MONO, 0},
@@ -70,7 +67,7 @@ static Tk_OptionSpec optionSpecs[] = {
 	DEF_SCALE_BORDER_WIDTH, -1, Tk_Offset(TkScale, borderWidth), 
         0, 0, 0},
     {TK_OPTION_STRING, "-command", "command", "Command",
-	DEF_SCALE_COMMAND, Tk_Offset(TkScale, commandPtr), -1,
+	DEF_SCALE_COMMAND, -1, Tk_Offset(TkScale, command),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_SCALE_CURSOR, -1, Tk_Offset(TkScale, cursor),
@@ -78,9 +75,6 @@ static Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_INT, "-digits", "digits", "Digits", 
 	DEF_SCALE_DIGITS, -1, Tk_Offset(TkScale, digits), 
         0, 0, 0},
-/*    {TK_CONFIG_CUSTOM, "-disabledtile", "disabledTile", "Tile", (char *) NULL,
-	Tk_Offset(TkScale, disabledTile), TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},*/
     {TK_OPTION_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, -1, 0, (ClientData) "-foreground", 0},
     {TK_OPTION_FONT, "-font", "font", "Font",
@@ -101,12 +95,10 @@ static Tk_OptionSpec optionSpecs[] = {
 	"HighlightThickness", DEF_SCALE_HIGHLIGHT_WIDTH, -1, 
 	Tk_Offset(TkScale, highlightWidth), 0, 0, 0},
     {TK_OPTION_STRING, "-label", "label", "Label",
-	DEF_SCALE_LABEL, Tk_Offset(TkScale, labelPtr), -1, 0, 0, 0},
+	DEF_SCALE_LABEL, -1, Tk_Offset(TkScale, label),
+	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-length", "length", "Length",
 	DEF_SCALE_LENGTH, -1, Tk_Offset(TkScale, length), 0, 0, 0},
-/*    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
-	Tk_Offset(TkScale, tsoffset), TK_CONFIG_DONT_SET_DEFAULT,
-	&offsetOption},*/
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient",
         DEF_SCALE_ORIENT, -1, Tk_Offset(TkScale, orient), 
         0, (ClientData) orientStrings, 0},
@@ -139,17 +131,11 @@ static Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_DOUBLE, "-tickinterval", "tickInterval", "TickInterval",
         DEF_SCALE_TICK_INTERVAL, -1, Tk_Offset(TkScale, tickInterval),
         0, 0, 0},
-/*    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(TkScale, tile), TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},*/
     {TK_OPTION_DOUBLE, "-to", "to", "To",
         DEF_SCALE_TO, -1, Tk_Offset(TkScale, toValue), 0, 0, 0},
     {TK_OPTION_COLOR, "-troughcolor", "troughColor", "Background",
         DEF_SCALE_TROUGH_COLOR, -1, Tk_Offset(TkScale, troughColorPtr),
         0, (ClientData) DEF_SCALE_TROUGH_MONO, 0},
-/*    {TK_CONFIG_CUSTOM, "-troughtile", "troughTile", "Tile", (char *) NULL,
-	Tk_Offset(TkScale, troughTile), TK_CONFIG_DONT_SET_DEFAULT,
-	&tileOption},*/
     {TK_OPTION_STRING, "-variable", "variable", "Variable",
 	DEF_SCALE_VARIABLE, Tk_Offset(TkScale, varNamePtr), -1,
 	TK_OPTION_NULL_OK, 0, 0},
@@ -196,8 +182,7 @@ static int		ScaleWidgetObjCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Obj *CONST objv[]));
 static void		ScaleWorldChanged _ANSI_ARGS_((
 			    ClientData instanceData));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
+static void		ScaleSetVariable _ANSI_ARGS_((TkScale *scalePtr));
 
 /*
  * The structure below defines scale class behavior by means of procedures
@@ -270,7 +255,7 @@ Tk_ScaleObjCmd(clientData, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    TkClassOptionObj(tkwin, "Scale", &objc, &objv);
+    Tk_SetClass(tkwin, "Scale");
     scalePtr = TkpCreateScale(tkwin);
 
     /*
@@ -279,78 +264,73 @@ Tk_ScaleObjCmd(clientData, interp, objc, objv)
      * (e.g. resource pointers).
      */
 
-    scalePtr->tkwin = tkwin;
-    scalePtr->display = Tk_Display(tkwin);
-    scalePtr->interp = interp;
-    scalePtr->widgetCmd = Tcl_CreateObjCommand(interp,
+    scalePtr->tkwin		= tkwin;
+    scalePtr->display		= Tk_Display(tkwin);
+    scalePtr->interp		= interp;
+    scalePtr->widgetCmd		= Tcl_CreateObjCommand(interp,
 	    Tk_PathName(scalePtr->tkwin), ScaleWidgetObjCmd,
 	    (ClientData) scalePtr, ScaleCmdDeletedProc);
-    scalePtr->optionTable = optionTable;
-    scalePtr->orient = ORIENT_VERTICAL;
-    scalePtr->width = 0;
-    scalePtr->length = 0;
-    scalePtr->value = 0.0;
-    scalePtr->varNamePtr = NULL;
-    scalePtr->fromValue = 0.0;
-    scalePtr->toValue = 0.0;
-    scalePtr->tickInterval = 0.0;
-    scalePtr->resolution = 1.0;
-    scalePtr->digits = 0;
-    scalePtr->bigIncrement = 0.0;
-    scalePtr->commandPtr = NULL;
-    scalePtr->repeatDelay = 0;
-    scalePtr->repeatInterval = 0;
-    scalePtr->labelPtr = NULL;
-    scalePtr->labelLength = 0;
-    scalePtr->state = TK_STATE_NORMAL;
-    scalePtr->borderWidth = 0;
-    scalePtr->bgBorder = NULL;
-    scalePtr->activeBorder = NULL;
-    scalePtr->sliderRelief = TK_RELIEF_RAISED;
-    scalePtr->troughColorPtr = NULL;
-    scalePtr->troughGC = None;
-    scalePtr->copyGC = None;
-    scalePtr->tkfont = NULL;
-    scalePtr->textColorPtr = NULL;
-    scalePtr->textGC = None;
-    scalePtr->relief = TK_RELIEF_FLAT;
-    scalePtr->highlightWidth = 0;
-    scalePtr->highlightBorder = NULL;
-    scalePtr->highlightColorPtr = NULL;
-    scalePtr->inset = 0;
-    scalePtr->sliderLength = 0;
-    scalePtr->showValue = 0;
-    scalePtr->horizLabelY = 0;
-    scalePtr->horizValueY = 0;
-    scalePtr->horizTroughY = 0;
-    scalePtr->horizTickY = 0;
-    scalePtr->vertTickRightX = 0;
-    scalePtr->vertValueRightX = 0;
-    scalePtr->vertTroughX = 0;
-    scalePtr->vertLabelX = 0;
-    scalePtr->cursor = None;
-    scalePtr->takeFocusPtr = NULL;
-    scalePtr->flags = NEVER_SET;
-    scalePtr->tile = scalePtr->activeTile = NULL;
-    scalePtr->disabledTile = scalePtr->troughTile = NULL;
-    scalePtr->activeTileGC = None;
+    scalePtr->optionTable	= optionTable;
+    scalePtr->orient		= ORIENT_VERTICAL;
+    scalePtr->width		= 0;
+    scalePtr->length		= 0;
+    scalePtr->value		= 0.0;
+    scalePtr->varNamePtr	= NULL;
+    scalePtr->fromValue		= 0.0;
+    scalePtr->toValue		= 0.0;
+    scalePtr->tickInterval	= 0.0;
+    scalePtr->resolution	= 1.0;
+    scalePtr->digits		= 0;
+    scalePtr->bigIncrement	= 0.0;
+    scalePtr->command		= NULL;
+    scalePtr->repeatDelay	= 0;
+    scalePtr->repeatInterval	= 0;
+    scalePtr->label		= NULL;
+    scalePtr->labelLength	= 0;
+    scalePtr->state		= STATE_NORMAL;
+    scalePtr->borderWidth	= 0;
+    scalePtr->bgBorder		= NULL;
+    scalePtr->activeBorder	= NULL;
+    scalePtr->sliderRelief	= TK_RELIEF_RAISED;
+    scalePtr->troughColorPtr	= NULL;
+    scalePtr->troughGC		= None;
+    scalePtr->copyGC		= None;
+    scalePtr->tkfont		= NULL;
+    scalePtr->textColorPtr	= NULL;
+    scalePtr->textGC		= None;
+    scalePtr->relief		= TK_RELIEF_FLAT;
+    scalePtr->highlightWidth	= 0;
+    scalePtr->highlightBorder	= NULL;
+    scalePtr->highlightColorPtr	= NULL;
+    scalePtr->inset		= 0;
+    scalePtr->sliderLength	= 0;
+    scalePtr->showValue		= 0;
+    scalePtr->horizLabelY	= 0;
+    scalePtr->horizValueY	= 0;
+    scalePtr->horizTroughY	= 0;
+    scalePtr->horizTickY	= 0;
+    scalePtr->vertTickRightX	= 0;
+    scalePtr->vertValueRightX	= 0;
+    scalePtr->vertTroughX	= 0;
+    scalePtr->vertLabelX	= 0;
+    scalePtr->fontHeight	= 0;
+    scalePtr->cursor		= None;
+    scalePtr->takeFocusPtr	= NULL;
+    scalePtr->flags		= NEVER_SET;
 
     TkSetClassProcs(scalePtr->tkwin, &scaleClass, (ClientData) scalePtr);
     Tk_CreateEventHandler(scalePtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    ScaleEventProc, (ClientData) scalePtr);
 
-    if (Tk_InitOptions(interp, (char *) scalePtr, optionTable, tkwin)
-	    != TCL_OK) {
+    if ((Tk_InitOptions(interp, (char *) scalePtr, optionTable, tkwin)
+	    != TCL_OK) ||
+	    (ConfigureScale(interp, scalePtr, objc - 2, objv + 2) != TCL_OK)) {
 	Tk_DestroyWindow(scalePtr->tkwin);
 	return TCL_ERROR;
     }
-    if (ConfigureScale(interp, scalePtr, objc - 2, objv + 2) != TCL_OK) {
-	Tk_DestroyWindow(scalePtr->tkwin);
-	return TCL_ERROR;
-    }
-    Tcl_SetStringObj(Tcl_GetObjResult(interp), Tk_PathName(scalePtr->tkwin),
-	    -1);
+
+    Tcl_SetResult(interp, Tk_PathName(scalePtr->tkwin), TCL_STATIC);
     return TCL_OK;
 }
 
@@ -446,9 +426,9 @@ ScaleWidgetObjCmd(clientData, interp, objc, objv)
 	    if (scalePtr->orient == ORIENT_VERTICAL) {
 	        x = scalePtr->vertTroughX + scalePtr->width/2
 		        + scalePtr->borderWidth;
-		y = TkpValueToPixel(scalePtr, value);
+		y = TkScaleValueToPixel(scalePtr, value);
 	    } else {
-	        x = TkpValueToPixel(scalePtr, value);
+	        x = TkScaleValueToPixel(scalePtr, value);
 		y = scalePtr->horizTroughY + scalePtr->width/2
                         + scalePtr->borderWidth;
 	    }
@@ -473,7 +453,7 @@ ScaleWidgetObjCmd(clientData, interp, objc, objv)
                         != TCL_OK)) {
 		    goto error;
 		}
-		value = TkpPixelToValue(scalePtr, x, y);
+		value = TkScalePixelToValue(scalePtr, x, y);
 	    }
 	    sprintf(buf, scalePtr->format, value);
 	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
@@ -514,8 +494,8 @@ ScaleWidgetObjCmd(clientData, interp, objc, objv)
 	    if (Tcl_GetDoubleFromObj(interp, objv[2], &value) != TCL_OK) {
 	        goto error;
 	    }
-	    if ((scalePtr->state != STATE_DISABLED)) {
-	      TkpSetScaleValue(scalePtr, value, 1, 1);
+	    if (scalePtr->state != STATE_DISABLED) {
+	      TkScaleSetValue(scalePtr, value, 1, 1);
 	    }
 	    break;
         } 
@@ -552,6 +532,13 @@ DestroyScale(memPtr)
 {
     register TkScale *scalePtr = (TkScale *) memPtr;
 
+    scalePtr->flags |= SCALE_DELETED;
+
+    Tcl_DeleteCommandFromToken(scalePtr->interp, scalePtr->widgetCmd);
+    if (scalePtr->flags & REDRAW_PENDING) {
+	Tcl_CancelIdleCall(TkpDisplayScale, (ClientData) scalePtr);
+    }
+
     /*
      * Free up all the stuff that requires special handling, then
      * let Tk_FreeOptions handle all the standard option-related
@@ -572,48 +559,12 @@ DestroyScale(memPtr)
     if (scalePtr->textGC != None) {
 	Tk_FreeGC(scalePtr->display, scalePtr->textGC);
     }
-    if (scalePtr->tile != NULL) {
-	Tk_FreeTile(scalePtr->tile);
-    }
-    if (scalePtr->activeTile != NULL) {
-	Tk_FreeTile(scalePtr->activeTile);
-    }
-    if (scalePtr->disabledTile != NULL) {
-	Tk_FreeTile(scalePtr->disabledTile);
-    }
-    if (scalePtr->troughTile != NULL) {
-	Tk_FreeTile(scalePtr->troughTile);
-    }
-    if (scalePtr->activeTileGC != None) {
-	Tk_FreeGC(scalePtr->display, scalePtr->activeTileGC);
-    }
     Tk_FreeConfigOptions((char *) scalePtr, scalePtr->optionTable,
 	    scalePtr->tkwin);
+    scalePtr->tkwin = NULL;
     TkpDestroyScale(scalePtr);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *  None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;           /* Not used */
-{
-    register TkScale *scalePtr = (TkScale *) clientData;
-
-    ConfigureScale(scalePtr->interp, scalePtr, 0, NULL);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -646,14 +597,14 @@ ConfigureScale(interp, scalePtr, objc, objv)
     Tk_SavedOptions savedOptions;
     Tcl_Obj *errorResult = NULL;
     int error;
-    char *label;
+    double oldValue = scalePtr->value;
 
     /*
      * Eliminate any existing trace on a variable monitored by the scale.
      */
 
     if (scalePtr->varNamePtr != NULL) {
-	Tcl_UntraceVar(interp, Tcl_GetString(scalePtr->varNamePtr), 
+	Tcl_UntraceVar(interp, Tcl_GetString(scalePtr->varNamePtr),
 		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		ScaleVarProc, (ClientData) scalePtr);
     }
@@ -686,15 +637,13 @@ ConfigureScale(interp, scalePtr, objc, objv)
 	 */
 
 	if (scalePtr->varNamePtr != NULL) {
-	    char *name;
 	    double value;
 	    Tcl_Obj *valuePtr;
 
-	    name = Tcl_GetString(scalePtr->varNamePtr);
-	    valuePtr = Tcl_GetVar2Ex(interp, name, NULL, TCL_GLOBAL_ONLY);
+	    valuePtr = Tcl_ObjGetVar2(interp, scalePtr->varNamePtr, NULL,
+		    TCL_GLOBAL_ONLY);
 	    if ((valuePtr != NULL) &&
-		    (Tcl_GetDoubleFromObj(interp, valuePtr, &value)
-		    == TCL_OK)) {
+		    (Tcl_GetDoubleFromObj(NULL, valuePtr, &value) == TCL_OK)) {
 		scalePtr->value = TkRoundToResolution(scalePtr, value);
 	    }
 	}
@@ -716,26 +665,13 @@ ConfigureScale(interp, scalePtr, objc, objv)
 	 */
 
 	if ((scalePtr->tickInterval < 0)
-	    ^     ((scalePtr->toValue - scalePtr->fromValue) <  0)) {
+		^ ((scalePtr->toValue - scalePtr->fromValue) <  0)) {
 	  scalePtr->tickInterval = -scalePtr->tickInterval;
 	}
 
-	/*
-	 * Set the scale value to itself;  all this does is to make sure
-	 * that the scale's value is within the new acceptable range for
-	 * the scale and reflect the value in the associated variable,
-	 * if any.
-	 */
-
 	ComputeFormat(scalePtr);
-	TkpSetScaleValue(scalePtr, scalePtr->value, 1, 1);
 
-	if (scalePtr->labelPtr != NULL) {
-	    label = Tcl_GetString(scalePtr->labelPtr);
-	    scalePtr->labelLength = strlen(label);
-	} else {
-	    scalePtr->labelLength = 0;
-	}
+	scalePtr->labelLength = scalePtr->label ? strlen(scalePtr->label) : 0;
 
 	Tk_SetBackgroundFromBorder(scalePtr->tkwin, scalePtr->bgBorder);
 
@@ -749,11 +685,33 @@ ConfigureScale(interp, scalePtr, objc, objv)
         Tk_FreeSavedOptions(&savedOptions);
     }
 
+    /*
+     * Set the scale value to itself;  all this does is to make sure
+     * that the scale's value is within the new acceptable range for
+     * the scale.  We don't set the var here because we need to make
+     * special checks for possibly changed varNamePtr.
+     */
+
+    TkScaleSetValue(scalePtr, scalePtr->value, 0, 1);
+
     /*
      * Reestablish the variable trace, if it is needed.
      */
 
     if (scalePtr->varNamePtr != NULL) {
+	Tcl_Obj *valuePtr;
+
+	/*
+	 * Set the associated variable only when the new value differs
+	 * from the current value, or the variable doesn't yet exist
+	 */
+	valuePtr = Tcl_ObjGetVar2(interp, scalePtr->varNamePtr, NULL,
+		TCL_GLOBAL_ONLY);
+	if ((valuePtr == NULL) || (scalePtr->value != oldValue)
+		|| (Tcl_GetDoubleFromObj(NULL, valuePtr, &oldValue) != TCL_OK)
+		|| (scalePtr->value != oldValue)) {
+	    ScaleSetVariable(scalePtr);
+	}
         Tcl_TraceVar(interp, Tcl_GetString(scalePtr->varNamePtr),
 	        TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 	        ScaleVarProc, (ClientData) scalePtr);
@@ -765,7 +723,7 @@ ConfigureScale(interp, scalePtr, objc, objv)
 	Tcl_DecrRefCount(errorResult);
 	return TCL_ERROR;
     } else {
-      return TCL_OK;
+	return TCL_OK;
     }
 }
 
@@ -794,49 +752,11 @@ ScaleWorldChanged(instanceData)
     XGCValues gcValues;
     GC gc;
     TkScale *scalePtr;
-    Pixmap pixmap;
-    Tk_Tile tile;
-    int flags;
 
     scalePtr = (TkScale *) instanceData;
 
-    if (scalePtr->state == TK_STATE_DISABLED) {
-	tile = scalePtr->disabledTile;
-    } else {
-	tile = scalePtr->tile;
-    }
-    Tk_SetTileChangedProc(scalePtr->tile, (Tk_TileChangedProc *) NULL,
-	    (ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(scalePtr->disabledTile, (Tk_TileChangedProc *) NULL,
-	    (ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(scalePtr->activeTile, (Tk_TileChangedProc *) NULL,
-	    (ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(tile, TileChangedProc,
-	    (ClientData) scalePtr, (Tk_Item *) NULL);
-    gcValues.graphics_exposures = False;
-    flags = GCGraphicsExposures;
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	flags |= GCTile|GCFillStyle;
-    }
-    gc = Tk_GetGC(scalePtr->tkwin, flags, &gcValues);
-    if (scalePtr->copyGC != None) {
-	Tk_FreeGC(scalePtr->display, scalePtr->copyGC);
-    }
-    scalePtr->copyGC = gc;
-
-    Tk_SetTileChangedProc(scalePtr->troughTile, TileChangedProc,
-	    (ClientData) scalePtr, (Tk_Item *) NULL);
-    if ((pixmap = Tk_PixmapOfTile(scalePtr->troughTile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	flags = GCTile|GCFillStyle;
-    } else {
-	gcValues.foreground = scalePtr->troughColorPtr->pixel;
-	flags = GCForeground;
-    }
-    gc = Tk_GetGC(scalePtr->tkwin, flags, &gcValues);
+    gcValues.foreground = scalePtr->troughColorPtr->pixel;
+    gc = Tk_GetGC(scalePtr->tkwin, GCForeground, &gcValues);
     if (scalePtr->troughGC != None) {
 	Tk_FreeGC(scalePtr->display, scalePtr->troughGC);
     }
@@ -850,21 +770,11 @@ ScaleWorldChanged(instanceData)
     }
     scalePtr->textGC = gc;
 
-    if (scalePtr->activeTile != NULL) {
-	tile = scalePtr->activeTile;
+    if (scalePtr->copyGC == None) {
+	gcValues.graphics_exposures = False;
+	scalePtr->copyGC = Tk_GetGC(scalePtr->tkwin, GCGraphicsExposures,
+	    &gcValues);
     }
-    if ((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	gc = Tk_GetGC(scalePtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else {
-	gc = None;
-    }
-    if (scalePtr->activeTileGC != None) {
-	Tk_FreeGC(scalePtr->display, scalePtr->activeTileGC);
-    }
-    scalePtr->activeTileGC = gc;
-
     scalePtr->inset = scalePtr->highlightWidth + scalePtr->borderWidth;
 
     /*
@@ -1012,7 +922,9 @@ ComputeScaleGeometry(scalePtr)
     char valueString[PRINT_CHARS];
     int tmp, valuePixels, x, y, extraSpace;
     Tk_FontMetrics fm;
-    char *label;
+
+    Tk_GetFontMetrics(scalePtr->tkfont, &fm);
+    scalePtr->fontHeight = fm.linespace + SPACING;
 
     /*
      * Horizontal scales are simpler than vertical ones because
@@ -1020,18 +932,17 @@ ComputeScaleGeometry(scalePtr)
      * handle them first and then quit.
      */
 
-    Tk_GetFontMetrics(scalePtr->tkfont, &fm);
-    if (!scalePtr->orient == ORIENT_VERTICAL) {
+    if (scalePtr->orient == ORIENT_HORIZONTAL) {
 	y = scalePtr->inset;
 	extraSpace = 0;
 	if (scalePtr->labelLength != 0) {
 	    scalePtr->horizLabelY = y + SPACING;
-	    y += fm.linespace + SPACING;
+	    y += scalePtr->fontHeight;
 	    extraSpace = SPACING;
 	}
 	if (scalePtr->showValue) {
 	    scalePtr->horizValueY = y + SPACING;
-	    y += fm.linespace + SPACING;
+	    y += scalePtr->fontHeight;
 	    extraSpace = SPACING;
 	} else {
 	    scalePtr->horizValueY = y;
@@ -1041,7 +952,7 @@ ComputeScaleGeometry(scalePtr)
 	y += scalePtr->width + 2*scalePtr->borderWidth;
 	if (scalePtr->tickInterval != 0) {
 	    scalePtr->horizTickY = y + SPACING;
-	    y += fm.linespace + 2*SPACING;
+	    y += scalePtr->fontHeight + SPACING;
 	}
 	Tk_GeometryRequest(scalePtr->tkwin,
 		scalePtr->length + 2*scalePtr->inset, y + scalePtr->inset);
@@ -1093,10 +1004,9 @@ ComputeScaleGeometry(scalePtr)
 	scalePtr->vertLabelX = 0;
     } else {
 	scalePtr->vertLabelX = x + fm.ascent/2;
-	label = Tcl_GetString(scalePtr->labelPtr);
 	x = scalePtr->vertLabelX + fm.ascent/2
-		+ Tk_TextWidth(scalePtr->tkfont, label,
-			scalePtr->labelLength);
+	    + Tk_TextWidth(scalePtr->tkfont, scalePtr->label,
+		    scalePtr->labelLength);
     }
     Tk_GeometryRequest(scalePtr->tkwin, x + scalePtr->inset,
 	    scalePtr->length + 2*scalePtr->inset);
@@ -1131,14 +1041,7 @@ ScaleEventProc(clientData, eventPtr)
     if ((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0)) {
 	TkEventuallyRedrawScale(scalePtr, REDRAW_ALL);
     } else if (eventPtr->type == DestroyNotify) {
-	if (scalePtr->tkwin != NULL) {
-	    scalePtr->tkwin = NULL;
-	    Tcl_DeleteCommandFromToken(scalePtr->interp, scalePtr->widgetCmd);
-	}
-	if (scalePtr->flags & REDRAW_ALL) {
-	    Tcl_CancelIdleCall(TkpDisplayScale, (ClientData) scalePtr);
-	}
-	Tcl_EventuallyFree((ClientData) scalePtr, DestroyScale);
+	DestroyScale((char *) clientData);
     } else if (eventPtr->type == ConfigureNotify) {
 	ComputeScaleGeometry(scalePtr);
 	TkEventuallyRedrawScale(scalePtr, REDRAW_ALL);
@@ -1191,8 +1094,8 @@ ScaleCmdDeletedProc(clientData)
      * destroys the widget.
      */
 
-    if (tkwin != NULL) {
-	scalePtr->tkwin = NULL;
+    if (!(scalePtr->flags & SCALE_DELETED)) {
+	scalePtr->flags |= SCALE_DELETED;
 	Tk_DestroyWindow(tkwin);
     }
 }
@@ -1226,7 +1129,8 @@ TkEventuallyRedrawScale(scalePtr, what)
 	    || !Tk_IsMapped(scalePtr->tkwin)) {
 	return;
     }
-    if ((scalePtr->flags & REDRAW_ALL) == 0) {
+    if (!(scalePtr->flags & REDRAW_PENDING)) {
+	scalePtr->flags |= REDRAW_PENDING;
 	Tcl_DoWhenIdle(TkpDisplayScale, (ClientData) scalePtr);
     }
     scalePtr->flags |= what;
@@ -1254,20 +1158,21 @@ TkRoundToResolution(scalePtr, value)
     TkScale *scalePtr;		/* Information about scale widget. */
     double value;		/* Value to round. */
 {
-    double rem, new;
+    double rem, new, tick;
 
     if (scalePtr->resolution <= 0) {
 	return value;
     }
-    new = scalePtr->resolution * floor(value/scalePtr->resolution);
+    tick = floor(value/scalePtr->resolution);
+    new = scalePtr->resolution * tick;
     rem = value - new;
     if (rem < 0) {
 	if (rem <= -scalePtr->resolution/2) {
-	    new -= scalePtr->resolution;
+	    new = (tick - 1.0) * scalePtr->resolution;
 	}
     } else {
 	if (rem >= scalePtr->resolution/2) {
-	    new += scalePtr->resolution;
+	    new = (tick + 1.0) * scalePtr->resolution;
 	}
     }
     return new;
@@ -1302,13 +1207,11 @@ ScaleVarProc(clientData, interp, name1, name2, flags)
     int flags;			/* Information about what happened. */
 {
     register TkScale *scalePtr = (TkScale *) clientData;
-    char *resultStr, *name;
+    char *resultStr;
     double value;
     Tcl_Obj *valuePtr;
     int result;
 
-    name = Tcl_GetString(scalePtr->varNamePtr);
-
     /*
      * If the variable is unset, then immediately recreate it unless
      * the whole interpreter is going away.
@@ -1316,17 +1219,17 @@ ScaleVarProc(clientData, interp, name1, name2, flags)
 
     if (flags & TCL_TRACE_UNSETS) {
 	if ((flags & TCL_TRACE_DESTROYED) && !(flags & TCL_INTERP_DESTROYED)) {
-	    Tcl_TraceVar(interp, name,
+	    Tcl_TraceVar(interp, Tcl_GetString(scalePtr->varNamePtr),
 		    TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		    ScaleVarProc, clientData);
 	    scalePtr->flags |= NEVER_SET;
-	    TkpSetScaleValue(scalePtr, scalePtr->value, 1, 0);
+	    TkScaleSetValue(scalePtr, scalePtr->value, 1, 0);
 	}
 	return (char *) NULL;
     }
 
     /*
-     * If we came here because we updated the variable (in TkpSetScaleValue),
+     * If we came here because we updated the variable (in TkScaleSetValue),
      * then ignore the trace.  Otherwise update the scale with the value
      * of the variable.
      */
@@ -1335,25 +1238,215 @@ ScaleVarProc(clientData, interp, name1, name2, flags)
 	return (char *) NULL;
     }
     resultStr = NULL;
-    valuePtr = Tcl_GetVar2Ex(interp, name, NULL, 
+    valuePtr = Tcl_ObjGetVar2(interp, scalePtr->varNamePtr, NULL, 
             TCL_GLOBAL_ONLY);
     result = Tcl_GetDoubleFromObj(interp, valuePtr, &value);
     if (result != TCL_OK) {
         resultStr = "can't assign non-numeric value to scale variable";
+	ScaleSetVariable(scalePtr);
     } else {
-      scalePtr->value = TkRoundToResolution(scalePtr, value);
-      
-      /*
-       * This code is a bit tricky because it sets the scale's value before
-       * calling TkpSetScaleValue.  This way, TkpSetScaleValue won't bother 
-       * to set the variable again or to invoke the -command.  However, it
-       * also won't redisplay the scale, so we have to ask for that
-       * explicitly.
-       */
+	scalePtr->value = TkRoundToResolution(scalePtr, value);
 
-      TkpSetScaleValue(scalePtr, scalePtr->value, 1, 0);
-      TkEventuallyRedrawScale(scalePtr, REDRAW_SLIDER);
+	/*
+	 * This code is a bit tricky because it sets the scale's value before
+	 * calling TkScaleSetValue.  This way, TkScaleSetValue won't bother 
+	 * to set the variable again or to invoke the -command.  However, it
+	 * also won't redisplay the scale, so we have to ask for that
+	 * explicitly.
+	 */
+
+	TkScaleSetValue(scalePtr, scalePtr->value, 1, 0);
     }
+    TkEventuallyRedrawScale(scalePtr, REDRAW_SLIDER);
 
     return resultStr;
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkScaleSetValue --
+ *
+ *	This procedure changes the value of a scale and invokes
+ *	a Tcl command to reflect the current position of a scale
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	A Tcl command is invoked, and an additional error-processing
+ *	command may also be invoked.  The scale's slider is redrawn.
+ *
+ *--------------------------------------------------------------
+ */
+
+void
+TkScaleSetValue(scalePtr, value, setVar, invokeCommand)
+    register TkScale *scalePtr;	/* Info about widget. */
+    double value;		/* New value for scale.  Gets adjusted
+				 * if it's off the scale. */
+    int setVar;			/* Non-zero means reflect new value through
+				 * to associated variable, if any. */
+    int invokeCommand;		/* Non-zero means invoked -command option
+				 * to notify of new value, 0 means don't. */
+{
+    value = TkRoundToResolution(scalePtr, value);
+    if ((value < scalePtr->fromValue)
+	    ^ (scalePtr->toValue < scalePtr->fromValue)) {
+	value = scalePtr->fromValue;
+    }
+    if ((value > scalePtr->toValue)
+	    ^ (scalePtr->toValue < scalePtr->fromValue)) {
+	value = scalePtr->toValue;
+    }
+    if (scalePtr->flags & NEVER_SET) {
+	scalePtr->flags &= ~NEVER_SET;
+    } else if (scalePtr->value == value) {
+	return;
+    }
+    scalePtr->value = value;
+    if (invokeCommand) {
+	scalePtr->flags |= INVOKE_COMMAND;
+    }
+    TkEventuallyRedrawScale(scalePtr, REDRAW_SLIDER);
+
+    if (setVar && scalePtr->varNamePtr) {
+	ScaleSetVariable(scalePtr);
+    }
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * ScaleSetVariable --
+ *
+ *	This procedure sets the variable associated with a scale, if any.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Other write traces on the variable will trigger.
+ *
+ *--------------------------------------------------------------
+ */
+
+static void
+ScaleSetVariable(scalePtr)
+    register TkScale *scalePtr;	/* Info about widget. */
+{
+    if (scalePtr->varNamePtr != NULL) {
+	char string[PRINT_CHARS];
+	sprintf(string, scalePtr->format, scalePtr->value);
+	scalePtr->flags |= SETTING_VAR;
+	Tcl_ObjSetVar2(scalePtr->interp, scalePtr->varNamePtr, NULL,
+		Tcl_NewStringObj(string, -1), TCL_GLOBAL_ONLY);
+	scalePtr->flags &= ~SETTING_VAR;
+    }
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkScalePixelToValue --
+ *
+ *	Given a pixel within a scale window, return the scale
+ *	reading corresponding to that pixel.
+ *
+ * Results:
+ *	A double-precision scale reading.  If the value is outside
+ *	the legal range for the scale then it's rounded to the nearest
+ *	end of the scale.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+double
+TkScalePixelToValue(scalePtr, x, y)
+    register TkScale *scalePtr;		/* Information about widget. */
+    int x, y;				/* Coordinates of point within
+					 * window. */
+{
+    double value, pixelRange;
+
+    if (scalePtr->orient == ORIENT_VERTICAL) {
+	pixelRange = Tk_Height(scalePtr->tkwin) - scalePtr->sliderLength
+		- 2*scalePtr->inset - 2*scalePtr->borderWidth;
+	value = y;
+    } else {
+	pixelRange = Tk_Width(scalePtr->tkwin) - scalePtr->sliderLength
+		- 2*scalePtr->inset - 2*scalePtr->borderWidth;
+	value = x;
+    }
+
+    if (pixelRange <= 0) {
+	/*
+	 * Not enough room for the slider to actually slide:  just return
+	 * the scale's current value.
+	 */
+
+	return scalePtr->value;
+    }
+    value -= scalePtr->sliderLength/2 + scalePtr->inset
+		+ scalePtr->borderWidth;
+    value /= pixelRange;
+    if (value < 0) {
+	value = 0;
+    }
+    if (value > 1) {
+	value = 1;
+    }
+    value = scalePtr->fromValue +
+		value * (scalePtr->toValue - scalePtr->fromValue);
+    return TkRoundToResolution(scalePtr, value);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkScaleValueToPixel --
+ *
+ *	Given a reading of the scale, return the x-coordinate or
+ *	y-coordinate corresponding to that reading, depending on
+ *	whether the scale is vertical or horizontal, respectively.
+ *
+ * Results:
+ *	An integer value giving the pixel location corresponding
+ *	to reading.  The value is restricted to lie within the
+ *	defined range for the scale.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkScaleValueToPixel(scalePtr, value)
+    register TkScale *scalePtr;		/* Information about widget. */
+    double value;			/* Reading of the widget. */
+{
+    int y, pixelRange;
+    double valueRange;
+
+    valueRange = scalePtr->toValue - scalePtr->fromValue;
+    pixelRange = ((scalePtr->orient == ORIENT_VERTICAL)
+	    ? Tk_Height(scalePtr->tkwin) : Tk_Width(scalePtr->tkwin))
+	- scalePtr->sliderLength - 2*scalePtr->inset - 2*scalePtr->borderWidth;
+    if (valueRange == 0) {
+	y = 0;
+    } else {
+	y = (int) ((value - scalePtr->fromValue) * pixelRange
+		  / valueRange + 0.5);
+	if (y < 0) {
+	    y = 0;
+	} else if (y > pixelRange) {
+	    y = pixelRange;
+	}
+    }
+    y += scalePtr->sliderLength/2 + scalePtr->inset + scalePtr->borderWidth;
+    return y;
+}
