@@ -13,15 +13,17 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkTest.c,v 1.5 1999/04/16 01:51:23 stanton Exp $
+ * RCS: @(#) $Id: tkTest.c,v 1.12 2000/04/04 08:09:15 hobbs Exp $
  */
 
+#include "tk.h"
+#include "tkText.h"
 #include "tkInt.h"
 #include "tkPort.h"
-#include "tkText.h"
 
 #ifdef __WIN32__
 #include "tkWinInt.h"
+#include "tkScrollbar.h"
 #endif
 
 #ifdef MAC_TCL
@@ -78,10 +80,17 @@ typedef struct TImageInstance {
  * The type record for test images:
  */
 
+#ifdef USE_OLD_IMAGE
+static int		ImageCreate _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *name, int argc, char **argv,
+			    Tk_ImageType *typePtr, Tk_ImageMaster master,
+			    ClientData *clientDataPtr));
+#else
 static int		ImageCreate _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *name, int argc, Tcl_Obj *CONST objv[],
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
+#endif
 static ClientData	ImageGet _ANSI_ARGS_((Tk_Window tkwin,
 			    ClientData clientData));
 static void		ImageDisplay _ANSI_ARGS_((ClientData clientData,
@@ -95,7 +104,7 @@ static void		ImageDelete _ANSI_ARGS_((ClientData clientData));
 
 static Tk_ImageType imageType = {
     "test",			/* name */
-    ImageCreate,		/* createProc */
+    (Tk_ImageCreateProc *) ImageCreate, /* createProc */
     ImageGet,			/* getProc */
     ImageDisplay,		/* displayProc */
     ImageFree,			/* freeProc */
@@ -159,9 +168,9 @@ static int		CBindingEvalProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, XEvent *eventPtr,
 			    Tk_Window tkwin, KeySym keySym));
 static void		CBindingFreeProc _ANSI_ARGS_((ClientData clientData));
-EXTERN int Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN int		Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 static int		ImageCmd _ANSI_ARGS_((ClientData dummy,
-			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST argv[]));
+			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestcbindCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int argc, char **argv));
 static int		TestbitmapObjCmd _ANSI_ARGS_((ClientData dummy,
@@ -217,13 +226,11 @@ static void		TrivialEventProc _ANSI_ARGS_((ClientData clientData,
  * External (platform specific) initialization routine:
  */
 
+extern int		TkplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
+
 #if !(defined(__WIN32__) || defined(MAC_TCL))
 #define TkplatformtestInit(x) TCL_OK
-#else
-extern int		TkplatformtestInit _ANSI_ARGS_((
-			    Tcl_Interp *interp));
 #endif
-
 
 /*
  *----------------------------------------------------------------------
@@ -304,12 +311,6 @@ Tktest_Init(interp)
     Tcl_CreateObjCommand(interp, "testwrapper", TestwrapperCmd,
 	    (ClientData) Tk_MainWindow(interp), (Tcl_CmdDeleteProc *) NULL);
 #endif
-
-#ifdef TCL_THREADS
-    if (TclThread_Init(interp) != TCL_OK) {
-	return TCL_ERROR;
-    }
-#endif
     
     /*
      * Create test image type.
@@ -1642,13 +1643,14 @@ TestfontObjCmd(clientData, interp, objc, objv)
  */
 
 	/* ARGSUSED */
+#ifdef USE_OLD_IMAGE
 static int
 ImageCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     Tcl_Interp *interp;		/* Interpreter for application containing
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    Tcl_Obj *CONST argv[];	/* Argument objects for options (doesn't
+    char **argv;		/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -1658,25 +1660,57 @@ ImageCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
 {
     TImageMaster *timPtr;
     char *varName;
-    int i, index;
-    static char *optionStrings[] = {
-	"-variable",	NULL
-    };
+    int i;
 
+    Tk_InitImageArgs(interp, argc, &argv);
     varName = "log";
     for (i = 0; i < argc; i += 2) {
-	if (Tcl_GetIndexFromObj(interp, argv[i], optionStrings, "option name", 0,
-		&index) != TCL_OK) {
+	if (strcmp(argv[i], "-variable") != 0) {
+	    Tcl_AppendResult(interp, "bad option name \"",
+		    argv[i], "\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
 	if ((i+1) == argc) {
 	    Tcl_AppendResult(interp, "no value given for \"",
-		    Tcl_GetStringFromObj(argv[i], NULL),
-		    "\" option", (char *) NULL);
+		    argv[i], "\" option", (char *) NULL);
 	    return TCL_ERROR;
 	}
-	varName = Tcl_GetStringFromObj(argv[i+1], NULL);
+	varName = argv[i+1];
     }
+#else
+static int
+ImageCreate(interp, name, objc, objv, typePtr, master, clientDataPtr)
+    Tcl_Interp *interp;		/* Interpreter for application containing
+				 * image. */
+    char *name;			/* Name to use for image. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument strings for options (doesn't
+				 * include image name or type). */
+    Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
+    Tk_ImageMaster master;	/* Token for image, to be used by us in
+				 * later callbacks. */
+    ClientData *clientDataPtr;	/* Store manager's token for image here;
+				 * it will be returned in later callbacks. */
+{
+    TImageMaster *timPtr;
+    char *varName;
+    int i;
+
+    varName = "log";
+    for (i = 0; i < objc; i += 2) {
+	if (strcmp(Tcl_GetString(objv[i]), "-variable") != 0) {
+	    Tcl_AppendResult(interp, "bad option name \"",
+		    Tcl_GetString(objv[i]), "\"", (char *) NULL);
+	    return TCL_ERROR;
+	}
+	if ((i+1) == objc) {
+	    Tcl_AppendResult(interp, "no value given for \"",
+		    Tcl_GetString(objv[i]), "\" option", (char *) NULL);
+	    return TCL_ERROR;
+	}
+	varName = Tcl_GetString(objv[i+1]);
+    }
+#endif
     timPtr = (TImageMaster *) ckalloc(sizeof(TImageMaster));
     timPtr->master = master;
     timPtr->interp = interp;
@@ -1686,7 +1720,7 @@ ImageCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     strcpy(timPtr->imageName, name);
     timPtr->varName = (char *) ckalloc((unsigned) (strlen(varName) + 1));
     strcpy(timPtr->varName, varName);
-    Tcl_CreateObjCommand(interp, name, ImageCmd, (ClientData) timPtr,
+    Tcl_CreateCommand(interp, name, ImageCmd, (ClientData) timPtr,
 	    (Tcl_CmdDeleteProc *) NULL);
     *clientDataPtr = (ClientData) timPtr;
     Tk_ImageChanged(master, 0, 0, 30, 15, 30, 15);
@@ -1716,33 +1750,37 @@ ImageCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Main window for application. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
-    Tcl_Obj *CONST argv[];		/* Argument objects. */
+    char **argv;			/* Argument strings. */
 {
     TImageMaster *timPtr = (TImageMaster *) clientData;
-    int x, y, width, height, index;
-    static char *optionStrings[] = {
-	"changed",	NULL
-    };
+    int x, y, width, height;
 
     if (argc < 2) {
-	Tcl_WrongNumArgs(interp, 1, argv, "option ?arg arg ...?");
+	Tcl_AppendResult(interp, "wrong # args: should be \"",
+		argv[0], "option ?arg arg ...?", (char *) NULL);
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, argv[1], optionStrings, "option", 0,
-	    &index) != TCL_OK) {
-	return TCL_ERROR;
-    }
-    if (argc != 8) {
-	Tcl_WrongNumArgs(interp, 2, argv,
-		"x y width height imageWidth imageHeight");
-	return TCL_ERROR;
-    }
-    if ((Tcl_GetIntFromObj(interp, argv[2], &x) != TCL_OK)
-	    || (Tcl_GetIntFromObj(interp, argv[3], &y) != TCL_OK)
-	    || (Tcl_GetIntFromObj(interp, argv[4], &width) != TCL_OK)
-	    || (Tcl_GetIntFromObj(interp, argv[5], &height) != TCL_OK)
-	    || (Tcl_GetIntFromObj(interp, argv[6], &timPtr->width) != TCL_OK)
-	    || (Tcl_GetIntFromObj(interp, argv[7], &timPtr->height) != TCL_OK)) {
+    if (strcmp(argv[1], "changed") == 0) {
+	if (argc != 8) {
+	    Tcl_AppendResult(interp, "wrong # args: should be \"",
+		    argv[0],
+		    " changed x y width height imageWidth imageHeight",
+		    (char *) NULL);
+	    return TCL_ERROR;
+	}
+	if ((Tcl_GetInt(interp, argv[2], &x) != TCL_OK)
+		|| (Tcl_GetInt(interp, argv[3], &y) != TCL_OK)
+		|| (Tcl_GetInt(interp, argv[4], &width) != TCL_OK)
+		|| (Tcl_GetInt(interp, argv[5], &height) != TCL_OK)
+		|| (Tcl_GetInt(interp, argv[6], &timPtr->width) != TCL_OK)
+		|| (Tcl_GetInt(interp, argv[7], &timPtr->height) != TCL_OK)) {
+	    return TCL_ERROR;
+	}
+	Tk_ImageChanged(timPtr->master, x, y, width, height, timPtr->width,
+		timPtr->height);
+    } else {
+	Tcl_AppendResult(interp, "bad option \"", argv[1],
+		"\": must be changed", (char *) NULL);
 	return TCL_ERROR;
     }
     Tk_ImageChanged(timPtr->master, x, y, width, height, timPtr->width,
@@ -2099,8 +2137,6 @@ TestmetricsCmd(clientData, interp, argc, argv)
 		"\": must be cxhscroll or cyvscroll", (char *) NULL);
 	return TCL_ERROR;
     }
-    Tcl_SetObjResult(Tcl_NewIntObject(
-	    GetSystemMetrics(((TkScrollbar *) winPtr->instanceData)->width))));
     return TCL_OK;
 }
 #endif
