@@ -12,9 +12,11 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWindow.c,v 1.8 1999/04/21 21:53:28 rjohnson Exp $
+ * RCS: @(#) $Id: tkWindow.c,v 1.16 2000/03/24 23:13:18 ericm Exp $
  */
 
+#define reserved284 tcl_SetMainLoop
+
 #include "tkPort.h"
 #include "tkInt.h"
 
@@ -25,6 +27,17 @@
 #include "tkWinInt.h"
 #endif
 
+#ifndef USE_TCL_STUBS
+extern TclStubs tclStubs;
+#define tclStubsPtr (&tclStubs)
+#endif
+
+/*
+ * Make sure that Tcl_SetMainLoop always references the stub pointer, not
+ * the real function.
+ */
+#undef Tcl_SetMainLoop
+#define Tcl_SetMainLoop ((void (*) _ANSI_ARGS_((VOID *))) ((tclStubsPtr)->tcl_SetMainLoop))
 
 typedef struct ThreadSpecificData {
     int numMainWindows;    /* Count of numver of main windows currently
@@ -104,18 +117,18 @@ static TkCmd commands[] = {
     {"bind",		Tk_BindCmd,		NULL,			1, 1},
     {"bindtags",	Tk_BindtagsCmd,		NULL,			1, 1},
     {"clipboard",	Tk_ClipboardCmd,	NULL,			0, 1},
-    {"destroy",		Tk_DestroyCmd,		NULL,			1, 1},
+    {"destroy",		NULL,			Tk_DestroyObjCmd,	1, 1},
     {"event",		NULL,			Tk_EventObjCmd,		1, 1},
     {"focus",		NULL,			Tk_FocusObjCmd,		1, 1},
     {"font",		NULL,			Tk_FontObjCmd,		1, 1},
     {"grab",		Tk_GrabCmd,		NULL,			0, 1},
     {"grid",		Tk_GridCmd,		NULL,			1, 1},
     {"image",		NULL,			Tk_ImageObjCmd,		1, 1},
-    {"lower",		Tk_LowerCmd,		NULL,			1, 1},
-    {"option",		Tk_OptionCmd,		NULL,			1, 1},
+    {"lower",		NULL,			Tk_LowerObjCmd,		1, 1},
+    {"option",		NULL,			Tk_OptionObjCmd,	1, 1},
     {"pack",		Tk_PackCmd,		NULL,			1, 1},
     {"place",		Tk_PlaceCmd,		NULL,			1, 1},
-    {"raise",		Tk_RaiseCmd,		NULL,			1, 1},
+    {"raise",		NULL,			Tk_RaiseObjCmd,		1, 1},
     {"selection",	Tk_SelectionCmd,	NULL,			0, 1},
     {"tk",		NULL,			Tk_TkObjCmd,		0, 1},
     {"tkwait",		Tk_TkwaitCmd,		NULL,			1, 1},
@@ -142,7 +155,7 @@ static TkCmd commands[] = {
     {"entry",		NULL,                   Tk_EntryObjCmd,		1, 0},
     {"frame",		NULL,			Tk_FrameObjCmd,		1, 1},
     {"label",		NULL,			Tk_LabelObjCmd,		1, 0},
-    {"listbox",		Tk_ListboxCmd,		NULL,			1, 1},
+    {"listbox",		NULL,			Tk_ListboxObjCmd,	1, 0},
     {"menubutton",	NULL,                   Tk_MenubuttonObjCmd,	1, 0},
     {"message",		Tk_MessageCmd,		NULL,			1, 1},
     {"radiobutton",	NULL,			Tk_RadiobuttonObjCmd,	1, 0},
@@ -217,7 +230,7 @@ static void		OpenIM _ANSI_ARGS_((TkDisplay *dispPtr));
 static void		UnlinkWindow _ANSI_ARGS_((TkWindow *winPtr));
 #ifdef TK_USE_INPUT_METHODS
 #ifdef I18N_IMPROVE
-static int		CanInitiateIm _ANSI_ARGS_((Tcl_Obj *locale));
+static int		CanInitiateIm _ANSI_ARGS_((void));
 static void		IMInstantiateCallback _ANSI_ARGS_ ((Display *display,
 				XPointer clientData, XPointer callData));
 static void		IMDestroyCallback _ANSI_ARGS_ ((XIM im, XPointer clientData, 
@@ -246,7 +259,6 @@ static void		IMDestroyCallback _ANSI_ARGS_ ((XIM im, XPointer clientData,
  *
  *----------------------------------------------------------------------
  */
-#undef Tk_CreatePhotoImageFormat
 
 static Tk_Window
 CreateTopLevelWindow(interp, parent, name, screenName)
@@ -276,15 +288,15 @@ CreateTopLevelWindow(interp, parent, name, screenName)
 	 * Create built-in image types.
 	 */
     
-	Tk_CreateNewImageType(&tkBitmapImageType);
-	Tk_CreateNewImageType(&tkPhotoImageType);
+	Tk_CreateImageType(&tkBitmapImageType);
+	Tk_CreateImageType(&tkPhotoImageType);
     
 	/*
 	 * Create built-in photo image formats.
 	 */
     
-	Tk_CreateNewPhotoImageFormat(&tkImgFmtGIF);
-	Tk_CreatePhotoImageFormat(&tkImgFmtPPM);
+	Tk_CreatePhotoImageFormat(&tkImgFmtGIF);
+	Tk_CreateOldPhotoImageFormat(&tkImgFmtPPM);
 
 	/*
 	 * Create exit handler to delete all windows when the application
@@ -474,6 +486,7 @@ GetScreen(interp, screenName, screenPtr)
             dispPtr->refCount = 0;
 	    strncpy(dispPtr->name, screenName, length);
 	    dispPtr->name[length] = '\0';
+	    dispPtr->useInputMethods = 0;
 	    OpenIM(dispPtr);
 	    TkInitXId(dispPtr);
 
@@ -646,6 +659,9 @@ TkAllocWindow(dispPtr, screenNum, parentPtr)
     winPtr->handlerList = NULL;
 #ifdef TK_USE_INPUT_METHODS
     winPtr->inputContext = NULL;
+#ifdef I18N_IMPROVE
+    Tcl_DStringInit(&(winPtr->composedDStr));
+#endif /* I18N_IMPROVE */
 #endif /* TK_USE_INPUT_METHODS */
     winPtr->tagPtr = NULL;
     winPtr->numTags = 0;
@@ -1296,6 +1312,10 @@ Tk_DestroyWindow(tkwin)
   	XDestroyIC(winPtr->inputContext);
 #endif /* I18N_IMPROVE */
     }
+#ifdef I18N_IMPROVE
+    Tcl_DStringFree(&(winPtr->composedDStr));
+    Tcl_DStringInit(&(winPtr->composedDStr));
+#endif /* I18N_IMPROVE */
 #endif /* TK_USE_INPUT_METHODS */
     if (winPtr->tagPtr != NULL) {
 	TkFreeBindingTags(winPtr);
@@ -2069,35 +2089,6 @@ Tk_SetClass(tkwin, className)
     TkOptionClassChanged(winPtr);
 }
 
-
-void TkClassOption(tkwin, name, argcp, argvp)
-    Tk_Window tkwin;
-    char *name;
-    int *argcp;
-    char ***argvp;
-{
-    if (((*argcp)>3) && !strcmp((*argvp)[2],"-class")) {
-	(*argcp) -= 2;
-	(*argvp) += 2;
-	name = (*argvp)[1];
-    }
-    Tk_SetClass(tkwin, name);
-}
-
-void TkClassOptionObj(tkwin, name, argcp, argvp)
-    Tk_Window tkwin;
-    char *name;
-    int *argcp;
-    Tcl_Obj * CONST **argvp;
-{
-    if (((*argcp)>3) && !strcmp(Tcl_GetStringFromObj((*argvp)[2], NULL), "-class")) {
-	(*argcp) -= 2;
-	(*argvp) += 2;
-	name = Tcl_GetStringFromObj((*argvp)[1], NULL);
-    }
-    Tk_SetClass(tkwin, name);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -2157,6 +2148,15 @@ Tk_NameToWindow(interp, pathName, tkwin)
 {
     Tcl_HashEntry *hPtr;
 
+    if (tkwin == NULL) {
+	/*
+	 * Either we're not really in Tk, or the main window was destroyed and
+	 * we're on our way out of the application
+	 */
+	Tcl_AppendResult(interp, "NULL main window", (char *)NULL);
+	return NULL;
+    }
+    
     hPtr = Tcl_FindHashEntry(&((TkWindow *) tkwin)->mainPtr->nameTable,
 	    pathName);
     if (hPtr == NULL) {
@@ -2312,9 +2312,6 @@ Tk_RestackWindow(tkwin, aboveBelow, other)
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
     TkWindow *otherPtr = (TkWindow *) other;
-    XWindowChanges changes;
-    unsigned int mask;
-
 
     /*
      * Special case:  if winPtr is a top-level window then just find
@@ -2322,8 +2319,6 @@ Tk_RestackWindow(tkwin, aboveBelow, other)
      * otherPtr without changing any of Tk's childLists.
      */
 
-    changes.stack_mode = aboveBelow;
-    mask = CWStackMode;
     if (winPtr->flags & TK_TOP_LEVEL) {
 	while ((otherPtr != NULL) && !(otherPtr->flags & TK_TOP_LEVEL)) {
 	    otherPtr = otherPtr->parentPtr;
@@ -2395,6 +2390,10 @@ Tk_RestackWindow(tkwin, aboveBelow, other)
      */
 
     if (winPtr->window != None) {
+	XWindowChanges changes;
+	unsigned int mask;
+
+	mask = CWStackMode;
 	changes.stack_mode = Above;
 	for (otherPtr = winPtr->nextPtr; otherPtr != NULL;
 		otherPtr = otherPtr->nextPtr) {
@@ -2480,12 +2479,29 @@ Tk_StrictMotif(tkwin)
 #ifdef I18N_IMPROVE
 
 static int
-CanInitiateIm(loc)
-     Tcl_Obj *loc;
+CanInitiateIm()
 {
     static int inited = 0;
     static int ret = 0;
-    char *locale = Tcl_GetString(loc);
+    char *locale;
+
+    /*
+     * Determine the current encoding from the LC_* or LANG environment
+     * variables.  We previously used setlocale() to determine the locale,
+     * but this does not work on some systems (e.g. Linux/i386 RH 5.0).
+     */
+
+    locale = getenv("LC_ALL");
+
+    if (locale == NULL || locale[0] == '\0') {
+	locale = getenv("LC_CTYPE");
+    }
+    if (locale == NULL || locale[0] == '\0') {
+	locale = getenv("LANG");
+    }
+    if (locale == NULL || locale[0] == '\0') {
+	return False;
+    }
 
     if (inited == 0) {
 	char *curSpec = setlocale(LC_ALL, NULL);
@@ -2618,6 +2634,8 @@ IMDestroyCallback(im, clientData, callData)
 		winPtr->inputContext != NULL) {
 		TkpDestroyIC(winPtr, 0);
 	    }
+	    Tcl_DStringFree(&(winPtr->composedDStr));
+	    Tcl_DStringInit(&(winPtr->composedDStr));
 	}
 	dispPtr->inputMethod = NULL;
 	dispPtr->lastFocusedIC = NULL;
@@ -2666,18 +2684,13 @@ OpenIM(dispPtr)
 
 #ifdef I18N_IMPROVE
     int styleFound = 0;
-    Tcl_Obj *imLocale = NULL;
-    Tcl_Obj *imEncoding = NULL;
 
     dispPtr->inputMethod = NULL;
     dispPtr->imEncoding = NULL;
     dispPtr->lastFocusedIC = NULL;
+    dispPtr->isComposed = 0;
 
-    TclpGetStartupLocaleEncoding(&imEncoding, &imLocale);
-    if (imEncoding == NULL || imLocale == NULL) {
-	return;
-    }
-    if (CanInitiateIm(imLocale) == False) {
+    if (CanInitiateIm() == False) {
 	return;
     }
 #endif /* I18N_IMPROVE */
@@ -2686,8 +2699,8 @@ OpenIM(dispPtr)
 #ifdef I18N_IMPROVE
 #ifdef HAVE_XIMREGINSTCB
 	/*
-	 * Maybe no IM server is available rite now.
-	 * Try to register instatntiate callback.
+	 * Maybe no IM server is available right now.
+	 * Try to register instantiate callback.
 	 */
 #ifdef NO_XIDPROC
 	XRegisterIMInstantiateCallback(dispPtr->display, NULL, NULL, NULL,
@@ -2722,7 +2735,7 @@ OpenIM(dispPtr)
 	/*
 	 * Create a Tcl_Encoding for XmbLookupString() conversion.
 	 */
-	dispPtr->imEncoding = Tcl_GetEncoding(NULL, Tcl_GetString(imEncoding));
+	dispPtr->imEncoding = Tcl_GetEncoding(NULL, Tcl_GetEncodingName(NULL));
 #ifdef XNDestroyCallback
 	dispPtr->destroyCallback.client_data = (XPointer)dispPtr;
 	dispPtr->destroyCallback.callback = (XIMProc)IMDestroyCallback;
@@ -2944,7 +2957,6 @@ Tk_SafeInit(interp)
 }
 
 
-extern TkStubs tkStubs;
 
 /*
  *----------------------------------------------------------------------
@@ -2968,21 +2980,10 @@ Initialize(interp)
 {
     char *p;
     int argc, code;
-    char **argv;
-    Tcl_Obj *args[20];
+    char **argv, *args[20];
     Tcl_DString class;
     ThreadSpecificData *tsdPtr;
     
-#if defined(__WIN32__) && defined(STATIC_BUILD) && !defined(NO_STATIC_INIT)
-    /*
-     * If we are in a statically linked executable, then we need to
-     * explicitly initialize the Windows function tables here since
-     * DllMain() will not be invoked.
-     */
-
-    TkWinXInit(GetModuleHandle(NULL));
-#endif
-
     /*
      * Ensure that we are getting the matching version of Tcl.  This is
      * really only an issue when Tk is loaded dynamically.
@@ -2992,10 +2993,6 @@ Initialize(interp)
         return TCL_ERROR;
     }
 
-    if (Tcl_SetMainLoop) {
-	Tcl_SetMainLoop(Tk_GetNumMainWindows);
-    }
-
     tsdPtr = (ThreadSpecificData *) 
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
@@ -3148,14 +3145,14 @@ Initialize(interp)
      * using the information parsed from argv, if any.
      */
 
-    args[0] = Tcl_NewStringObj("toplevel", -1);
-    args[1] = Tcl_NewStringObj(".", -1);
-    args[2] = Tcl_NewStringObj("-class", -1);
-    args[3] = Tcl_NewStringObj(Tcl_DStringValue(&class), -1);
+    args[0] = "toplevel";
+    args[1] = ".";
+    args[2] = "-class";
+    args[3] = Tcl_DStringValue(&class);
     argc = 4;
     if (display != NULL) {
-	args[argc] = Tcl_NewStringObj("-screen", -1);
-	args[argc+1] = Tcl_NewStringObj(display, -1);
+	args[argc] = "-screen";
+	args[argc+1] = display;
 	argc += 2;
 
 	/*
@@ -3169,20 +3166,20 @@ Initialize(interp)
 	}
     }
     if (colormap != NULL) {
-	args[argc] = Tcl_NewStringObj("-colormap", -1);
-	args[argc+1] = Tcl_NewStringObj(colormap, -1);
+	args[argc] = "-colormap";
+	args[argc+1] = colormap;
 	argc += 2;
         colormap = NULL;
     }
     if (use != NULL) {
-	args[argc] = Tcl_NewStringObj("-use", -1);
-	args[argc+1] = Tcl_NewStringObj(use, -1);
+	args[argc] = "-use";
+	args[argc+1] = use;
 	argc += 2;
         use = NULL;
     }
     if (visual != NULL) {
-	args[argc] = Tcl_NewStringObj("-visual", -1);
-	args[argc+1] = Tcl_NewStringObj(visual, -1);
+	args[argc] = "-visual";
+	args[argc+1] = visual;
 	argc += 2;
         visual = NULL;
     }
@@ -3233,7 +3230,7 @@ Initialize(interp)
 #endif
 
     Tk_InitStubs(interp, TK_VERSION, 1);
-#endif
+#endif /* __WIN32__ */
 
     /*
      * Invoke platform-specific initialization.
@@ -3241,6 +3238,10 @@ Initialize(interp)
 
     code = TkpInit(interp);
 
+    if (Tcl_SetMainLoop) {
+	Tcl_SetMainLoop(Tk_GetNumMainWindows);
+    }
+
     done:
     if (argv != NULL) {
 	ckfree((char *) argv);
