@@ -3,7 +3,7 @@
 # This file defines the default bindings for Tk entry widgets and provides
 # procedures that help in implementing those bindings.
 #
-# RCS: @(#) $Id: entry.tcl,v 1.7 1999/08/09 16:52:06 hobbs Exp $
+# RCS: @(#) $Id: entry.tcl,v 1.11 2000/01/06 02:22:24 hobbs Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
@@ -50,7 +50,7 @@ bind Entry <<Copy>> {
 bind Entry <<Paste>> {
     global tcl_platform
     catch {
-      if {[string compare $tcl_platform(platform) "unix"]} {
+	if {[string compare $tcl_platform(platform) "unix"]} {
 	    catch {
 		%W delete sel.first sel.last
 	    }
@@ -202,7 +202,7 @@ bind Entry <Escape> {# nothing}
 bind Entry <Return> {# nothing}
 bind Entry <KP_Enter> {# nothing}
 bind Entry <Tab> {# nothing}
-if {![string compare $tcl_platform(platform) "macintosh"]} {
+if {[string equal $tcl_platform(platform) "macintosh"]} {
 	bind Entry <Command-KeyPress> {# nothing}
 }
 
@@ -217,7 +217,7 @@ if {[string compare $tcl_platform(platform) "windows"]} {
 # input method configuration binding (unix only)
 if {![string compare $tcl_platform(platform) "unix"]} {
     if {[string compare [info commands imconfigure] {}]} {
-	bind Text <FocusIn> {catch {imconfigure %W}}
+	bind Entry <FocusIn> {catch {imconfigure %W}}
     }
 }
 
@@ -343,7 +343,7 @@ proc tkEntryButton1 {w x} {
     set tkPriv(pressX) $x
     $w icursor [tkEntryClosestGap $w $x]
     $w selection from insert
-    if {![string compare [$w cget -state] "normal"]} {focus $w}
+    if {[string equal [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryMouseSelect --
@@ -413,7 +413,7 @@ proc tkEntryPaste {w x} {
 
     $w icursor [tkEntryClosestGap $w $x]
     catch {$w insert insert [selection get -displayof $w]}
-    if {![string compare [$w cget -state] "normal"]} {focus $w}
+    if {[string equal [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryAutoScan --
@@ -437,7 +437,7 @@ proc tkEntryAutoScan {w} {
 	$w xview scroll -2 units
 	tkEntryMouseSelect $w $x
     }
-    set tkPriv(afterId) [after 50 tkEntryAutoScan $w]
+    set tkPriv(afterId) [after 50 [list tkEntryAutoScan $w]]
 }
 
 # tkEntryKeySelect --
@@ -470,7 +470,7 @@ proc tkEntryKeySelect {w new} {
 # s -		The string to insert (usually just a single character)
 
 proc tkEntryInsert {w s} {
-    if {![string compare $s ""]} {
+    if {[string equal $s ""]} {
 	return
     }
     catch {
@@ -516,15 +516,8 @@ proc tkEntryBackspace w {
 
 proc tkEntrySeeInsert w {
     set c [$w index insert]
-    set left [$w index @0]
-    if {$left > $c} {
+    if {($c < [$w index @0]) || ($c > [$w index @[winfo width $w]])} {
 	$w xview $c
-	return
-    }
-    set x [winfo width $w]
-    while {([$w index @$x] <= $c) && ($left < $c)} {
-	incr left
-	$w xview $left
     }
 }
 
@@ -543,98 +536,6 @@ proc tkEntrySetCursor {w pos} {
     tkEntrySeeInsert $w
 }
 
-# tkValidate --
-# This procedure validates particular types of numbers/formats
-#
-# Arguments:
-# type		- The type of validation (alphabetic, alphanumeric, date,
-#		hex, integer, numeric, real).  Date is always strict.
-# val		- The value to be validated
-#
-# Switches:
-# -strict	- enable more precise (strict) pattern matching on number
-#
-# Example use:	tkValidate real 55e-5
-#		tkValidate -strict integer -505
-#
-
-proc tkValidate {args} {
-  set strict 0
-  if [string match "-s*" [lindex $args 0]] {
-    set strict 1
-    set args [lreplace $args 0 0]
-  }
-
-  if {[llength $args] == 2} {
-    set type [lindex $args 0]
-    set val  [lindex $args 1]
-  } else {
-    error "called \"tkValidate\" with improper arguments\
-	\nmust be 'tkValidate ?-strict? type value'"    
-  }
-
-  switch -glob -- $type {
-    alphab* {
-      #alphabetic
-      if {$strict} {
-	return [regexp -nocase {^[a-z]+$} $val]
-      } else {
-	return [regexp -nocase {^[a-z]*$} $val]
-      }
-    }
-    alphan* {
-      #alphanumeric
-      if {$strict} {
-	return [regexp -nocase {^[a-z0-9]+$} $val]
-      } else {
-	return [regexp -nocase {^[a-z0-9]*$} $val]
-      }
-    }
-    d* {
-      #date
-      return [expr ![catch {clock scan $val}]]
-    }
-    h* {
-      #hexadecimal
-      if {$strict} {
-	return -nocase [regexp {^(0x)?[0-9a-f]+$} $val]
-      } else {
-	return -nocase [regexp {^(0x)?[0-9a-f]*$} $val]
-      }
-    }
-    i* {
-      #integer
-      if {$strict} {
-	return [regexp {^[-+]?[0-9]+$} $val]
-      } else {
-	return [regexp {^[-+]?[0-9]*$} $val]
-      }
-    }
-    n* {
-      #numeric
-      if {$strict} {
-	return [regexp {^[0-9]+$} $val]
-      } else {
-	return [regexp {^[0-9]*$} $val]
-      }
-    }
-    r* {
-      #real
-      if {$strict} {
-	return [regexp -nocase \
-	    {^[-+]?([0-9]+\.?[0-9]*|[0-9]*\.?[0-9]+)(e[-+]?[0-9]+)?$} $val]
-      } else {
-	return [regexp -nocase \
-	    {^[-+]?[0-9]*\.?[0-9]*([0-9]\.?e[-+]?[0-9]*)?$} $val]
-      }
-    }
-    default {
-      error "called \"tkValidate\" with unknown type \"$type\":\
-	\nmust be alphabetic, alphanumeric, date, hexadecimal, integer, numeric or real"
-    }
-  }
-}
-
 # tkEntryTranspose -
 # This procedure implements the "transpose" function for entry widgets.
 # It tranposes the characters on either side of the insertion cursor,
@@ -670,7 +571,7 @@ proc tkEntryTranspose w {
 # w -		The entry window in which the cursor is to move.
 # start -	Position at which to start search.
 
-if {![string compare $tcl_platform(platform) "windows"]}  {
+if {[string equal $tcl_platform(platform) "windows"]}  {
     proc tkEntryNextWord {w start} {
 	set pos [tcl_endOfWord [$w get] [$w index $start]]
 	if {$pos >= 0} {
@@ -716,9 +617,9 @@ proc tkEntryPreviousWord {w start} {
 
 proc tkEntryGetSelection {w} {
     set entryString [string range [$w get] [$w index sel.first] \
-                       [expr [$w index sel.last] - 1]]
-    if {[$w cget -show] != ""} {
-      regsub -all . $entryString [string index [$w cget -show] 0] entryString
+	    [expr {[$w index sel.last] - 1}]]
+    if {[string compare [$w cget -show] ""]} {
+	regsub -all . $entryString [string index [$w cget -show] 0] entryString
     }
     return $entryString
 }
