@@ -11,7 +11,7 @@
 # Copyright (c) 1998-1999 by Scriptics Corporation.
 # All rights reserved.
 # 
-# RCS: @(#) $Id: defs.tcl,v 1.4 1999/04/20 19:20:03 hershey Exp $
+# RCS: @(#) $Id: defs.tcl,v 1.7 1999/12/14 06:53:12 hobbs Exp $
 
 # Initialize wish shell
 
@@ -57,17 +57,12 @@ namespace eval tcltest {
 	namespace export $proc
     }
 
-    # ::tcltest::verbose defaults to "b"
-
-    variable verbose "b"
-
-    # match defaults to the empty list
-
-    variable match {}
-
-    # skip defaults to the empty list
-
-    variable skip {}
+    # setup ::tcltest default vars
+    foreach {var default} {verbose b match {} skip {}} {
+	if {![info exists $var]} {
+	    variable $var $default
+	}
+    }
 
     # Tests should not rely on the current working directory.
     # Files that are part of the test suite should be accessed relative to
@@ -119,6 +114,7 @@ namespace eval tcltest {
 
     variable ::tcltest::mainThread 1
     if {[info commands testthread] != {}} {
+	puts "Tk with threads enabled is known to have problems with X"
 	set ::tcltest::mainThread [testthread names]
     }
 }
@@ -224,6 +220,18 @@ proc ::tcltest::initConfig {} {
 	if {[string match {{22 3 6 15} {31 18 [34] 15}} $x] == 0} {
 	    set ::tcltest::testConfig(fonts) 0
 	}
+
+	# Test to see if we have are running Unix apps on Exceed,
+	# which won't return font failures (Windows-like), which is
+	# not what we want from ann X server (other Windows X servers
+	# operate as expected)
+
+	set ::tcltest::testConfig(noExceed) 1
+	if {$::tcltest::testConfig(unixOnly) && \
+		[catch {font actual "\{xyz"}] == 0} {
+	    puts "Running X app on Exceed, skipping problematic font tests..."
+	    set ::tcltest::testConfig(noExceed) 0
+	}
     }
 
     # Skip empty tests
@@ -998,22 +1006,20 @@ if {[info exists tk_version]} {
 	    cleanupbg
 	}
 	
-	# The following code segment cannot be run on Windows in Tk8.1b2
-	# This bug is logged as a pipe bug (bugID 1495).
+	# The following code segment cannot be run on Windows prior
+	# to Tk 8.1b3 due to a channel I/O bug (bugID 1495).
 
 	global tcl_platform
-	if {$tcl_platform(platform) != "windows"} {
-	    set ::tcltest::fd [open "|[list $::tcltest::tktest -geometry +0+0 -name tktest] $args" r+]
-	    puts $::tcltest::fd "puts foo; flush stdout"
-	    flush $::tcltest::fd
-	    if {[gets $::tcltest::fd data] < 0} {
-		error "unexpected EOF from \"$::tcltest::tktest\""
-	    }
-	    if {[string compare $data foo]} {
-		error "unexpected output from background process \"$data\""
-	    }
-	    fileevent $::tcltest::fd readable bgReady
+	set ::tcltest::fd [open "|[list $::tcltest::tktest -geometry +0+0 -name tktest] $args" r+]
+	puts $::tcltest::fd "puts foo; flush stdout"
+	flush $::tcltest::fd
+	if {[gets $::tcltest::fd data] < 0} {
+	    error "unexpected EOF from \"$::tcltest::tktest\""
 	}
+	if {[string compare $data foo]} {
+	    error "unexpected output from background process \"$data\""
+	}
+	fileevent $::tcltest::fd readable bgReady
     }
     
     # Send a command to the background process, catching errors and
