@@ -5,12 +5,12 @@
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  * Copyright (c) 1994 Software Research Associates, Inc.
- * Copyright (c) 1998 by Scriptics Corporation.
+ * Copyright (c) 1998-2000 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinX.c,v 1.7 1999/11/19 22:00:19 hobbs Exp $
+ * RCS: @(#) $Id: tkWinX.c,v 1.10 2000/04/19 01:06:51 ericm Exp $
  */
 
 #include "tkWinInt.h"
@@ -21,12 +21,6 @@
 
 #include <zmouse.h>
 
-/*
- * Definitions of extern variables supplied by this file.
- */
-
-static int tkPlatformId;
-
 /*
  * Declarations of static variables used in this file.
  */
@@ -35,6 +29,7 @@ static char winScreenName[] = ":0"; /* Default name of windows display. */
 static HINSTANCE tkInstance;        /* Application instance handle. */
 static int childClassInitialized;   /* Registered child class? */
 static WNDCLASS childClass;	    /* Window class for child windows. */
+static int tkPlatformId;	    /* version of Windows platform */
 
 TCL_DECLARE_MUTEX(winXMutex)
 
@@ -46,6 +41,7 @@ TCL_DECLARE_MUTEX(winXMutex)
 typedef struct ThreadSpecificData {
     TkDisplay *winDisplay;       /* TkDisplay structure that *
 				  *  represents Windows screen. */
+    int updatingClipboard;	/* If 1, we are updating the clipboard */
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
@@ -84,16 +80,14 @@ TkGetServerInfo(interp, tkwin)
     Tk_Window tkwin;		/* Token for window;  this selects a
 				 * particular display and server. */
 {
-    char buffer[50];
-    OSVERSIONINFO info;
+    char buffer[60];
+    OSVERSIONINFO os;
 
-    info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&info);
-    sprintf(buffer, "Windows %d.%d %d ", info.dwMajorVersion,
-	    info.dwMinorVersion, info.dwBuildNumber);
-    Tcl_AppendResult(interp, buffer,
-	    (info.dwPlatformId == VER_PLATFORM_WIN32s) ? "Win32s" : "Win32",
-	    (char *) NULL);
+    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    GetVersionEx(&os);
+    sprintf(buffer, "Windows %d.%d %d Win32", os.dwMajorVersion,
+	    os.dwMinorVersion, os.dwBuildNumber);
+    Tcl_SetResult(interp, buffer, TCL_VOLATILE);
 }
 
 /*
@@ -138,11 +132,7 @@ void
 TkWinXInit(hInstance)
     HINSTANCE hInstance;
 {
-    OSVERSIONINFO info;
-
-    info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&info);
-    tkPlatformId = info.dwPlatformId;
+    OSVERSIONINFO os;
 
     if (childClassInitialized != 0) {
 	return;
@@ -151,6 +141,10 @@ TkWinXInit(hInstance)
 
     tkInstance = hInstance;
 
+    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    GetVersionEx(&os);
+    tkPlatformId = os.dwPlatformId;
+
     /*
      * When threads are enabled, we cannot use CLASSDC because
      * threads will then write into the same device context.
@@ -222,6 +216,32 @@ TkWinXCleanup(hInstance)
     TkWinWmCleanup(hInstance);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinGetPlatformId --
+ *
+ *	Determines whether running under NT, 95, or Win32s, to allow 
+ *	runtime conditional code.
+ *
+ * Results:
+ *	The return value is one of:
+ *	    VER_PLATFORM_WIN32s		Win32s on Windows 3.1. 
+ *	    VER_PLATFORM_WIN32_WINDOWS	Win32 on Windows 95.
+ *	    VER_PLATFORM_WIN32_NT	Win32 on Windows NT
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int		
+TkWinGetPlatformId()
+{
+    return tkPlatformId;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -244,6 +264,7 @@ TkGetDefaultScreenName(interp, screenName)
     Tcl_Interp *interp;		/* Not used. */
     char *screenName;		/* If NULL, use default string. */
 {
+    TkWinXInit(GetModuleHandle(NULL));
     if ((screenName == NULL) || (screenName[0] == '\0')) {
 	screenName = winScreenName;
     }
@@ -378,13 +399,14 @@ TkpOpenDisplay(display_name)
     screen->white_pixel = RGB(255, 255, 255);
     screen->black_pixel = RGB(0, 0, 0);
 
-    display->screens = screen;
-    display->nscreens = 1;
-    display->default_screen = 0;
+    display->screens		= screen;
+    display->nscreens		= 1;
+    display->default_screen	= 0;
     screen->cmap = XCreateColormap(display, None, screen->root_visual,
 	    AllocNone);
     tsdPtr->winDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
     tsdPtr->winDisplay->display = display;
+    tsdPtr->updatingClipboard = FALSE;
     return tsdPtr->winDisplay;
 }
 
@@ -663,6 +685,8 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 {
     XEvent event;
     TkWindow *winPtr = (TkWindow *)Tk_HWNDToWindow(hwnd);
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (!winPtr || winPtr->window == None) {
 	return;
@@ -727,6 +751,13 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	}
 
 	case WM_DESTROYCLIPBOARD:
+	    if (tsdPtr->updatingClipboard == TRUE) {
+		/*
+		 * We want to avoid this event if we are the ones that caused
+		 * this event.
+		 */
+		return;
+	    }
 	    event.type = SelectionClear;
 	    event.xselectionclear.selection =
 		Tk_InternAtom((Tk_Window)winPtr, "CLIPBOARD");
@@ -935,7 +966,7 @@ GetState(message, wParam, lParam)
 		mask = ControlMask;
 		break;
 	    case VK_MENU:
-		mask = Mod2Mask;
+		mask = ALT_MASK;
 		break;
 	    case VK_CAPITAL:
 		if (message == WM_SYSKEYDOWN || message == WM_KEYDOWN) {
@@ -1134,16 +1165,11 @@ TkpGetMS()
 /*
  *----------------------------------------------------------------------
  *
- * TkWinGetPlatformId --
+ * TkWinUpdatingClipboard --
  *
- *	Determines whether running under NT, 95, or Win32s, to allow 
- *	runtime conditional code.
  *
  * Results:
- *	The return value is one of:
- *	    VER_PLATFORM_WIN32s		Win32s on Windows 3.1. 
- *	    VER_PLATFORM_WIN32_WINDOWS	Win32 on Windows 95.
- *	    VER_PLATFORM_WIN32_NT	Win32 on Windows NT
+ *	Number of milliseconds.
  *
  * Side effects:
  *	None.
@@ -1151,9 +1177,11 @@ TkpGetMS()
  *----------------------------------------------------------------------
  */
 
-int		
-TkWinGetPlatformId()
+void
+TkWinUpdatingClipboard(int mode)
 {
-    return tkPlatformId;
-}
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
+    tsdPtr->updatingClipboard = mode;
+}
